unit xButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls, Extctrls;

type TxButtonState=(xbsNormal,xbsUp,xbsDown);

type TParentControl = class(TWinControl);

type
  TxButton = class(TGraphicControl)
  private
    fBitmap:TBitmap;
    fState:TxButtonState;
    fOnClick,fOnMouseDown,fOnMouseUp:TMouseEvent;
    procedure SetBitmap(bmp:TBitmap);
    function MouseOverControl:boolean;
  protected
    procedure Paint; override;
    procedure MouseLeave(var msg : tmessage); message CM_MOUSELEAVE;
    procedure MouseEnter(var msg : tmessage); message CM_MOUSEENTER;
    procedure MouseMove(Shift:TShiftState; X,Y:Integer); override;
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure Click; override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    property Bitmap:TBitmap read fBitmap write SetBitmap;
    property OnClick:TMouseEvent read fOnClick write fOnClick;
    property OnMouseDown:TMouseEvent read fOnMouseDown write fOnClick;
    property OnMouseUp:TMouseEvent read fOnMouseUp write fOnClick;
    property OnMouseMove;
    property Visible;
    property Enabled; 
    property Hint;
    property ShowHint;
  end;

procedure Register;
procedure CopyParentImage(Control: TControl; Dest: TCanvas);

implementation
{------------------------------------------------------------------------------}
constructor TxButton.Create(AOwner:TComponent);
begin
   inherited;
   Width:=25; Height:=25; fState:=xbsNormal;
   fBitmap:=TBitmap.Create;
   fBitmap.TransparentMode:=tmAuto;
   fBitmap.Transparent:=True;
end;
{------------------------------------------------------------------------------}
destructor TxButton.Destroy;
begin
   fBitmap.Free;
   inherited;
end;
{------------------------------------------------------------------------------}
procedure TxButton.Paint;
var B:TBitmap;
begin
 B:=TBitmap.Create; B.Width:=Width; B.Height:=Height;
 B.Canvas.Brush.Color:=clBlack;
 B.Canvas.CopyMode:=cmSrcCopy;
 If fBitmap.Empty Then B.Canvas.FrameRect(Rect(1,1,Width-1,Height-1)) Else
 Case fState of
 xbsNormal: Begin // Normal ----------------------
             CopyParentImage(Self,B.Canvas);
             B.Canvas.StretchDraw(Rect(0,0,Width,Height),fBitmap);
//             B.Canvas.CopyRect(Rect(0,0,Width,Height),fBitmap.Canvas,Rect(0,0,fBitmap.Width,fBitmap.Height));
            End;
 xbsUp:     Begin // Up --------------------------
             CopyParentImage(Self,B.Canvas);
             B.Canvas.StretchDraw(Rect(0,0,Width,Height),fBitmap);
//             B.Canvas.CopyRect(Rect(0,0,Width,Height),fBitmap.Canvas,Rect(0,0,fBitmap.Width,fBitmap.Height));
             B.Canvas.Pen.Color:=clWhite;
             B.Canvas.MoveTo(0,Height-1); B.Canvas.LineTo(0,0); B.Canvas.LineTo(Width-1,0);
             B.Canvas.Pen.Color:=clGray;
             B.Canvas.LineTo(Width-1,Height-1); B.Canvas.LineTo(0,Height-1);
            End;
 xbsDown:   Begin // Down ------------------------
             CopyParentImage(Self,B.Canvas);
             B.Canvas.StretchDraw(Rect(1,1,Width+1,Height+1),fBitmap);
             B.Canvas.Pen.Color:=clBlack;
             B.Canvas.MoveTo(0,Height-1); B.Canvas.LineTo(0,0); B.Canvas.LineTo(Width-1,0);
             B.Canvas.Pen.Color:=clWhite;
             B.Canvas.LineTo(Width-1,Height-1); B.Canvas.LineTo(0,Height-1);
            End;
 End;
 Canvas.Draw(0,0,B);
 B.Free;
end;
{------------------------------------------------------------------------------}
procedure TxButton.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
begin
 If Button=mbLeft Then Begin
   fState:=xbsDown; Paint;
   If Assigned(fOnMouseDown) Then fOnMouseDown(Self,Button,Shift,X,Y);
 End;
end;
{------------------------------------------------------------------------------}
procedure TxButton.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
begin
 If Assigned(fOnMouseUp) Then fOnMouseUp(Self,Button,Shift,X,Y);
 If MouseOverControl
 Then Begin
  fState:=xbsUp; Paint;
  If Assigned(fOnClick) and (Button=mbLeft) Then fOnClick(Self,Button,Shift,X,Y);
  If not MouseOverControl Then Begin fState:=xbsNormal; Paint; End;
 End
 Else Begin
  fState:=xbsNormal; Paint;
 End;
end;
{------------------------------------------------------------------------------}
procedure TxButton.MouseMove(Shift:TShiftState; X,Y:Integer);
begin

end;
{------------------------------------------------------------------------------}
procedure TxButton.Click;
begin
// If MouseOverControl Then fState:=xbsUp Else fState:=xbsNormal;
// Paint;
end;
{------------------------------------------------------------------------------}
procedure TxButton.MouseLeave(var msg : tmessage);
begin
 fState:=xbsNormal; Paint;
end;
{------------------------------------------------------------------------------}
procedure TxButton.MouseEnter(var msg : tmessage);
begin
 fState:=xbsUp; Paint;
end;
{------------------------------------------------------------------------------}
procedure TxButton.SetBitmap(bmp:TBitmap);
begin
 fBitmap.Assign(bmp);
 fBitmap.TransparentMode:=tmAuto;
 fBitmap.Transparent:=True;
 Paint;
end;
{------------------------------------------------------------------------------}
function TxButton.MouseOverControl:boolean;
var XY:TPoint;
    x,y:integer;
begin
 GetCursorPos(XY);  X:=Parent.Left+Left; Y:=Parent.Top+Top;
 If (XY.X>=X) and (XY.Y>=Y) and (XY.X<=X+Width) and (XY.Y<=Y+Height)
 Then Result:=True Else Result:=False;
end;
{------------------------------------------------------------------------------}
procedure Register;
begin
 RegisterComponents('XRat', [TxButton]);
end;
{------------------------------------------------------------------------------}
procedure CopyParentImage(Control: TControl; Dest: TCanvas);
var
  I, Count, X, Y, SaveIndex: Integer;
  DC: HDC;
  R, SelfR, CtlR: TRect;
begin
  if (Control = nil) or (Control.Parent = nil) then Exit;
  Count := Control.Parent.ControlCount;
  DC := Dest.Handle;
{$IFDEF WIN32}
  with Control.Parent do ControlState := ControlState + [csPaintCopy];
  try
{$ENDIF}
    with Control do begin
      SelfR := Bounds(Left, Top, Width, Height);
      X := -Left; Y := -Top;
    end;
    { Copy parent control image }
    SaveIndex := SaveDC(DC);
    try
      SetViewportOrgEx(DC, X, Y, nil);
      IntersectClipRect(DC, 0, 0, Control.Parent.ClientWidth,
        Control.Parent.ClientHeight);
      with TParentControl(Control.Parent) do begin
        Perform(WM_ERASEBKGND, DC, 0);
        PaintWindow(DC);
      end;
    finally
      RestoreDC(DC, SaveIndex);
    end;
    { Copy images of graphic controls }
    for I := 0 to Count - 1 do begin
      if Control.Parent.Controls[I] = Control then Break
      else if (Control.Parent.Controls[I] <> nil) and
        (Control.Parent.Controls[I] is TGraphicControl) then
      begin
        with TGraphicControl(Control.Parent.Controls[I]) do begin
          CtlR := Bounds(Left, Top, Width, Height);
          if Bool(IntersectRect(R, SelfR, CtlR)) and Visible then begin
{$IFDEF WIN32}
            ControlState := ControlState + [csPaintCopy];
{$ENDIF}
            SaveIndex := SaveDC(DC);
            try
              SaveIndex := SaveDC(DC);
              SetViewportOrgEx(DC, Left + X, Top + Y, nil);
              IntersectClipRect(DC, 0, 0, Width, Height);
              Perform(WM_PAINT, DC, 0);
            finally
              RestoreDC(DC, SaveIndex);
{$IFDEF WIN32}
              ControlState := ControlState - [csPaintCopy];
{$ENDIF}
            end;
          end;
        end;
      end;
    end;
{$IFDEF WIN32}
  finally
    with Control.Parent do ControlState := ControlState - [csPaintCopy];
  end;
{$ENDIF}
end;
{------------------------------------------------------------------------------}

end.
