unit xFileMenu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, RxMenus;

type TxPopupClick = procedure(Filename:string) of object;

type
  TxFileMenu = class(TRxPopupMenu)
  private
    fOnPopup:TxPopupClick;
    fOnAdd, fOnOrg:TNotifyEvent;
    procedure MakeSubMenu(Path:string; Parent:TMenuItem);
    procedure PopupClick(Sender: TObject);
  protected
    { Protected declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  public
    { Public declarations }
   procedure OpenDir(Path:string);
   procedure ClearItems;
  published
   property OnPopupClick:TxPopupClick read fOnPopup write fOnPopup;
   property OnAddClick:TNotifyEvent read fOnAdd write fOnAdd;
   property OnOrganizeClick:TNotifyEvent read fOnOrg write fOnOrg;
    { Published declarations }
  end;

// Tag :
// 0 - Dir
// 1 - PLS
// 2 - M3U
// 3 - Service Items (add and organze)

procedure Register;
Function NormDir(Path:string):string;

implementation

{==============================================================================}
constructor TxFileMenu.Create(AOwner: TComponent);
begin
 inherited;
end;
{==============================================================================}
destructor TxFileMenu.Destroy;
begin
 inherited;
end;
{==============================================================================}
procedure TxFileMenu.OpenDir(Path:string);
var SubItem: TMenuItem;
begin
 ClearItems;
 Items.Caption:=Path;
 SubItem:=TMenuItem.Create(Self); SubItem.Tag:=3; SubItem.Caption:='Add To Favorites...';
 SubItem.OnClick:=fOnAdd; Items.Add(SubItem);
 SubItem:=TMenuItem.Create(Self); SubItem.Tag:=3; SubItem.Caption:='Organize Favorites...';
 SubItem.OnClick:=fOnOrg; Items.Add(SubItem);
 SubItem:=TMenuItem.Create(Self); SubItem.Tag:=3; SubItem.Caption:='-'; Items.Add(SubItem);
 makeSubMenu(Path,Items);
end;
{==============================================================================}
procedure TxFileMenu.ClearItems;
begin
 While Items.Count>0 Do Items.Delete(0);
end;
{==============================================================================}
procedure TxFileMenu.PopupClick(Sender: TObject);
var s:string;
{------------------------------------------------------------------------------}
function GetCaption(Parent:TMenuItem):string;
begin
 If Parent=Items Then result:=Parent.Caption Else result:=GetCaption(Parent.Parent)+'\'+Parent.Caption;
end;
{------------------------------------------------------------------------------}
begin
 s:=GetCaption(TMenuItem(Sender));
 If TMenuItem(Sender).Tag=2 Then s:=s+'.m3u' Else s:=s+'.pls';
 If Assigned(fOnPopup) Then fOnPopup(s);
end;
{==============================================================================}
procedure TxFileMenu.MakeSubMenu(Path:string; Parent:TMenuItem);
var SubItem: TMenuItem;
    Dirs:TSearchRec;
    s:string;
    Res,i:integer;
    iList:TStringList;
Begin
iList:=TStringList.Create;
// Directories
res:=FindFirst(NormDir(Path)+'*.*',faReadOnly+faHidden+faArchive+faDirectory,Dirs);
While res=0 Do
Begin
 If ((Dirs.Attr and faDirectory)<>0) and not ((Dirs.Name='.') or (Dirs.Name='..'))
 Then iList.Add(Dirs.Name);
 res:=FindNext(Dirs);
End;
FindClose(Dirs);

iList.Sort;
For i:=0 To iList.Count-1 Do
Begin
 SubItem:=TMenuItem.Create(Self);
 SubItem.Caption:=iList.Strings[i];
 SubItem.Tag:=0;
 Parent.Add(SubItem);
 makeSubMenu(NormDir(Path)+iList.Strings[i]+'\',SubItem);
End;

iList.Clear;

// Files
res:=FindFirst(NormDir(Path)+'*.*',faReadOnly+faHidden+faArchive+faDirectory,Dirs);
While res=0 Do
Begin
 s:=Lowercase(ExtractFileExt(Dirs.Name));
 If not ((Dirs.Attr and faDirectory)<>0) and ((s='.pls') or (s='.m3u'))
 Then iList.Add(Dirs.Name);
 res:=FindNext(Dirs);
End;
FindClose(Dirs);

iList.Sort;

For i:=0 To iList.Count-1 Do
Begin
 SubItem:=TMenuItem.Create(Self);
 SubItem.Caption:=Copy(iList.Strings[i],1,Length(iList.Strings[i])-4);
 s:=Lowercase(ExtractFileExt(iList.Strings[i]));
 If s='.m3u' Then SubItem.Tag:=2 Else SubItem.Tag:=1;
 SubItem.OnClick:=PopupClick;
 Parent.Add(SubItem);
End;

iList.Free;
End;
{==============================================================================}
Function NormDir(Path:string):string;
Begin
 If Length(Path)=0 Then Begin Result:=''; exit; End;
 If Path[Length(Path)]='\' Then Result:=Path Else Result:=Path+'\';
End;
{==============================================================================}
procedure Register;
begin
  RegisterComponents('XRat', [TxFileMenu]);
end;
{==============================================================================}
end.
