unit xLed;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls, Extctrls,
  xGraphLib;

type
  TxLed = class(TGraphicControl)
  private
    fBmp1:TBitmap;
    fBmp2:TBitmap;
    fBmp3:TBitmap;
    fBmpN:TBitmap;
    fTimer:TTimer;
    fTransparent:boolean;
    fTranspMode:TTransparentMode;
    fTranspColor:TColor;
    idx:byte;
    fBackground:boolean;
    procedure SetAnimated(f:boolean);
    function GetAnimated:boolean;
    procedure SetInterval(i:Cardinal);
    function GetInterval:Cardinal;
    procedure SetBmp1(bmp:TBitmap);
    procedure SetBmp2(bmp:TBitmap);
    procedure SetBmp3(bmp:TBitmap);
    procedure SetBmpN(bmp:TBitmap);
    procedure SetIndex(i:byte);
    procedure SetTransparent(f:boolean);
    procedure SetTranspMode(m:TTransparentMode);
    procedure SetTranspColor(c:TColor);
    procedure OnTimer(Sender:TObject);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    property Bitmap_1:TBitmap read fBmp1 write SetBmp1;
    property Bitmap_2:TBitmap read fBmp2 write SetBmp2;
    property Bitmap_3:TBitmap read fBmp3 write SetBmp3;
    property Bitmap_N:TBitmap read fBmpN write SetBmpN;
    property Transparent:boolean read fTransparent write SetTransparent;
    property TransparentMode:TTransparentMode read fTranspMode write SetTranspMode;
    property TransparentColor:TColor read fTranspColor write SetTranspColor;
    property BitmapIndex:byte read idx write SetIndex;
    property Animated:boolean read GetAnimated write SetAnimated;
    property Interval:Cardinal read GetInterval write SetInterval;
    property Hint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
  end;

procedure Register;

implementation
{------------------------------------------------------------------------------}
constructor TxLed.Create(AOwner:TComponent);
begin
   inherited;
   Width:=25; Height:=25;
   Idx:=1;
   fBmp1:=TBitmap.Create; fBmp2:=TBitmap.Create;
   fBmp3:=TBitmap.Create; fBmpN:=TBitmap.Create;
   fTimer:=TTimer.Create(Self);
   fTimer.Enabled:=False;
   fTimer.Interval:=1000;
   fBackground:=True; fTransparent:=False; fTranspColor:=clBlack; fTranspMode:=tmAuto;
   fTimer.OnTimer:=OnTimer;
end;
{------------------------------------------------------------------------------}
destructor TxLed.Destroy;
begin
   fBmp1.Free; fBmp2.Free; fBmp3.Free; fBmpN.Free;
   fTimer.Free;
   inherited;
end;
{------------------------------------------------------------------------------}
procedure TxLed.Paint;
var fBitmap:TBitmap;
begin
 fBitmap:=TBitmap.Create; fBitmap.Width:=ClientWidth; fBitmap.Height:=ClientHeight;
 fBitmap.Canvas.Brush.Color:=clBlack; fBitmap.Canvas.CopyMode:=cmSrcCopy;
 If (not (csDesigning in ComponentState)) and (not (csLoading in ComponentState)) and fTransparent
 Then CopyParentImage(Self,fBitmap.Canvas);
 If fTimer.Enabled and fBackground and (not fBmpN.Empty)
 Then fBitmap.Canvas.StretchDraw(Rect(0,0,ClientWidth,ClientHeight),fBmpN)
 Else
 Case Idx of
  1: If not fBmp1.Empty Then fBitmap.Canvas.StretchDraw(Rect(0,0,ClientWidth,ClientHeight),fBmp1) Else fBitmap.Canvas.FrameRect(Rect(0,0,ClientWidth,ClientHeight));
  2: If not fBmp2.Empty Then fBitmap.Canvas.StretchDraw(Rect(0,0,ClientWidth,ClientHeight),fBmp2) Else fBitmap.Canvas.FrameRect(Rect(0,0,ClientWidth,ClientHeight));
  3: If not fBmp3.Empty Then fBitmap.Canvas.StretchDraw(Rect(0,0,ClientWidth,ClientHeight),fBmp3) Else fBitmap.Canvas.FrameRect(Rect(0,0,ClientWidth,ClientHeight));
  0: If not fBmpN.Empty Then fBitmap.Canvas.StretchDraw(Rect(0,0,ClientWidth,ClientHeight),fBmpN) Else fBitmap.Canvas.FrameRect(Rect(0,0,ClientWidth,ClientHeight));
 End;
 Canvas.CopyMode:=cmSrcCopy;
 Canvas.Draw(0,0,fBitmap);
 fBitmap.Free;
end;
{------------------------------------------------------------------------------}
procedure TxLed.OnTimer(Sender:TObject);
begin
 fBackground:=not fBackground; Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetBmp1(bmp:TBitmap);
begin
 fBmp1.Assign(bmp); Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetBmp2(bmp:TBitmap);
begin
 fBmp2.Assign(bmp); Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetBmp3(bmp:TBitmap);
begin
 fBmp3.Assign(bmp); Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetBmpN(bmp:TBitmap);
begin
 fBmpN.Assign(bmp); Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetTransparent(f:boolean);
begin
 fTransparent:=f;
 fBmp1.Transparent:=fTransparent;
 fBmp2.Transparent:=fTransparent;
 fBmp3.Transparent:=fTransparent;
 fBmpN.Transparent:=fTransparent;
 Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetTranspMode(m:TTransparentMode);
begin
 fBmp1.TransparentMode:=m;
 fBmp2.TransparentMode:=m;
 fBmp3.TransparentMode:=m;
 fBmpN.TransparentMode:=m;
 fTranspMode:=m;
 Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetTranspColor(c:TColor);
begin
 If fTranspMode=tmAuto
 Then Begin fTranspColor:=c; exit; End;
 fBmp1.TransparentColor:=c;
 fBmp2.TransparentColor:=c;
 fBmp3.TransparentColor:=c;
 fBmpN.TransparentColor:=c;
 Paint;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetIndex(i:byte);
begin
 If i<4 Then Begin idx:=i; Paint; End;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetAnimated(f:boolean);
begin
 fBackground:=True; fTimer.Enabled:=f; Paint;
end;
{------------------------------------------------------------------------------}
function TxLed.GetAnimated:boolean;
begin
 Result:=fTimer.Enabled;
end;
{------------------------------------------------------------------------------}
procedure TxLed.SetInterval(i:Cardinal);
begin
 fTimer.Interval:=i;
end;
{------------------------------------------------------------------------------}
function TxLed.GetInterval:Cardinal;
begin
 Result:=fTimer.Interval;
end;
{------------------------------------------------------------------------------}
procedure Register;
begin
 RegisterComponents('XRat', [TxLed]);
end;
{------------------------------------------------------------------------------}
end.
