unit xScroller;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Extctrls, xGraphLib;

type
  TxScroller = class(TGraphicControl)
  private
    fTimer:TTimer;
    fBar,fSlider:TBitmap;
    fValue,fShift,fZoom,LastYpos,SliderYshift:integer;
    fOnChange,fOnUp,fOnDown,fOnPageDown,fOnPageUp:TNotifyEvent;
    fTransp,MoveSlider:boolean;
    fTranspMode:TTransparentMode;
    fTranspColor:TColor;
    procedure SetValue(v:integer);
    procedure SetBar(b:TBitmap);
    procedure SetSlider(b:TBitmap);
    procedure SetTransp(b:boolean);
    procedure SetTranspMode(m:TTransparentMode);
    procedure SetTranspColor(c:TColor);
    procedure SetZoom(z:integer);
    procedure OnTimer(Sender:TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseMove(Shift:TShiftState; X,Y:Integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    property OnChange:TNotifyEvent read fOnChange write fOnChange;
    property OnLineDown:TNotifyEvent read fOnDown write fOnDown;
    property OnLineUp:TNotifyEvent read fOnUp write fOnUp;
    property OnPageDown:TNotifyEvent read fOnPageDown write fOnPageDown;
    property OnPageUp:TNotifyEvent read fOnPageUp write fOnPageUp;
    property Value:integer read fValue write SetValue default 500;
    property BackgroundBitmap:TBitmap read fBar write SetBar;
    property SliderBitmap:TBitmap read fSlider write SetSlider;
    property Transparent:boolean read fTransp write SetTransp;
    property TransparentMode:TTransparentMode read fTranspMode write SetTranspMode;
    property TransparentColor:TColor read fTranspColor write SetTranspColor;
    property Zoom:integer read fZoom write SetZoom;
    property Enabled;
    property Hint;
    property ShowHint;
  end;

procedure Register;

implementation
{==============================================================================}
constructor TxScroller.Create(AOwner:TComponent);
begin
 inherited;
 Width:=15; Height:=80; fValue:=500; fShift:=0; fZoom:=1;
 LastYpos:=0; SliderYshift:=0; MoveSlider:=False;
 fBar:=TBitmap.Create; fSlider:=TBitmap.Create;
 fTransp:=False; fTranspMode:=tmAuto; fTranspColor:=clBlack;
 fTimer:=TTimer.Create(Self); fTimer.Enabled:=False; fTimer.Interval:=50;
 fTimer.OnTimer:=OnTimer; 
end;
{==============================================================================}
destructor TxScroller.Destroy;
begin
 fBar.Free; fSlider.Free; fTimer.Free;
 inherited;
end;
{==============================================================================}
procedure TxScroller.Paint;
var fRealBar,fRealSlider,fBitmap:TBitmap;
    i,Pos:integer;
begin
 Width:=15*fZoom;
 fRealBar:=TBitmap.Create; fRealBar.Width:=15*fZoom; fRealBar.Height:=80*fZoom;
 fBitmap:=TBitmap.Create; fBitmap.Width:=15*fZoom; fBitmap.Height:=Height;
 fRealSlider:=TBitmap.Create; fRealSlider.Width:=15*fZoom; fRealSlider.Height:=30*fZoom;
 fRealBar.Canvas.CopyMode:=cmSrcCopy; fBitmap.Canvas.CopyMode:=cmSrcCopy;
 fRealSlider.Canvas.CopyMode:=cmSrcCopy;

 If fBar.Empty // Make 'real sized' bar bitmap (15 x 80)
 Then Begin fRealBar.Canvas.Brush.Color:=clBlack; fRealBar.Canvas.FrameRect(Rect(1,1,fRealBar.Width-1,fRealBar.Height-1)); End
 Else Begin fRealBar.Canvas.StretchDraw(Rect(0,0,fRealBar.Width,fRealBar.Height),fBar); End;

 If (not (csDesigning in ComponentState)) and (not (csLoading in ComponentState)) and fTransp
 Then CopyParentImage(Self,fBitmap.Canvas);

 // Draw top part of bitmap
 fBitmap.Canvas.CopyRect(Rect(0,0,15*fZoom,25*fZoom),fRealBar.Canvas,Rect(0,0,15*fZoom,25*fZoom));

 // Draw middle part of bitmap
 For i:=0 To (Height-50*fZoom) div 30*fZoom
 Do Begin
  fBitmap.Canvas.CopyRect(Rect(0,25*fZoom+i*30*fZoom,15*fZoom,55*fZoom+i*30*fZoom),fRealBar.Canvas,Rect(0,25*fZoom,15*fZoom,55*fZoom));
 End;

 // Draw bottom part of bitmap
 fBitmap.Canvas.CopyRect(Rect(0,Height-25*fZoom,15*fZoom,Height),fRealBar.Canvas,Rect(0,55*fZoom,15*fZoom,80*fZoom));

 If not fSlider.Empty
 Then Begin
  fRealSlider.Canvas.StretchDraw(Rect(0,0,fRealSlider.Width,fRealBar.Height),fSlider);
  Pos:=Round((Height-62*fZoom)*(fValue/1000)); Pos:=Pos+16*fZoom; 
  fBitmap.Canvas.Draw(0,Pos,fRealSlider);
 End;

 Canvas.CopyMode:=cmSrcCopy;
 Canvas.Draw(0,0,fBitmap);

 fBitmap.Free; fRealBar.Free; fRealSlider.Free;
end;
{==============================================================================}
procedure TxScroller.OnTimer(Sender:TObject);
var Pos:integer;
begin
 If fShift=3 Then Begin If Assigned(fOnUp) Then fOnUp(Self); exit; End;
 If fShift=-3 Then Begin If Assigned(fOnDown) Then fOnDown(Self); exit; End;

 If fShift=14
 Then Begin
  Pos:=Round((Height-62*fZoom)*(fValue/1000)); Pos:=Pos+16*fZoom;
  If LastYpos>=Pos Then Begin fTimer.Enabled:=True; fShift:=0; exit; End;
  If Assigned(fOnPageUp) Then fOnPageUp(Self);
  exit;
 End;

 If fShift=-14
 Then Begin
  Pos:=Round((Height-62*fZoom)*(fValue/1000)); Pos:=Pos+16*fZoom;
  If LastYpos<=(Pos+30*fZoom) Then Begin fTimer.Enabled:=True; fShift:=0; exit; End;
  If Assigned(fOnPageDown) Then fOnPageDown(Self);
  exit;
 End;

 If fShift<0 Then Dec(fShift); If fShift>0 Then Inc(fShift);
end;
{==============================================================================}
procedure TxScroller.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
var Pos:integer;
begin
 If Button<>mbLeft Then exit;
 If y<(16*fZoom)
 Then Begin If Assigned(fOnUp) Then fOnUp(Self); fShift:=1; fTimer.Enabled:=True; exit; End;
 If y>(Height-16*fZoom)
 Then Begin If Assigned(fOnDown) Then fOnDown(Self); fShift:=-1; fTimer.Enabled:=True; exit; End;

 Pos:=Round((Height-62*fZoom)*(fValue/1000)); Pos:=Pos+16*fZoom;
 LastYpos:=y;

 If y<Pos
 Then Begin
  If Assigned(fOnPageUp) Then fOnPageUp(Self);
  fShift:=10; fTimer.Enabled:=True; exit;
 End;

 If y>(Pos+30*fZoom)
 Then Begin
  If Assigned(fOnPageDown) Then fOnPageDown(Self);
  fShift:=-10; fTimer.Enabled:=True; exit;
 End;

 SliderYshift:=y-Pos; MoveSlider:=True; 
end;
{==============================================================================}
procedure TxScroller.MouseMove(Shift:TShiftState; X,Y:Integer);
var Pos:integer;
begin
 If ssLeft in Shift Then LastYpos:=y;
 If not MoveSlider Then exit;
 Pos:=y-SliderYshift; fValue:=Round((Pos-16*fZoom)/(Height-62*fZoom)*1000);
 If fValue<0 Then fValue:=0; If fValue>1000 Then fValue:=1000; Paint;
 If Assigned(fOnChange) Then fOnChange(Self);
end;
{==============================================================================}
procedure TxScroller.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
begin
 If Button<>mbLeft Then exit;
 MoveSlider:=False;
 If fShift<>0 Then Begin fTimer.Enabled:=True; fShift:=0; End;
end;
{==============================================================================}
procedure TxScroller.SetValue(v:integer);
begin
 If MoveSlider Then exit;
 If v<0 Then v:=0; If v>1000 Then v:=1000;
 fValue:=v; Paint; 
end;
{==============================================================================}
procedure TxScroller.SetBar(b:TBitmap);
begin
 fBar.Assign(b); Paint;
end;
{==============================================================================}
procedure TxScroller.SetSlider(b:TBitmap);
begin
 fSlider.Assign(b); Paint;
end;
{==============================================================================}
procedure TxScroller.SetTransp(b:boolean);
begin
 fBar.Transparent:=b; fSlider.Transparent:=b; fTransp:=b; Paint;
end;
{==============================================================================}
procedure TxScroller.SetTranspMode(m:TTransparentMode);
begin
 fBar.TransparentMode:=m; fSlider.TransparentMode:=m; fTranspMode:=m; Paint;
end;
{==============================================================================}
procedure TxScroller.SetTranspColor(c:TColor);
begin
 fTranspColor:=c; If fTranspMode=tmAuto Then exit;
 fBar.TransparentColor:=c; fSlider.TransparentColor:=c; Paint;
end;
{==============================================================================}
procedure TxScroller.SetZoom(z:integer);
begin
 If z<1 Then exit; fZoom:=z; Paint;
end;
{==============================================================================}
procedure Register;
begin
 RegisterComponents('XRat', [TxScroller]);
end;
{==============================================================================}
end.
