unit xSlider;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Extctrls, xGraphLib;

type
  TxSlider = class(TGraphicControl)
  private
    fBar,fSlider:TBitmap;
    fValue,fSliderWidth,fShift:integer;
    fOnChange,fOnShow,fOnEndShow:TNotifyEvent;
    fTransp:boolean;
    fTranspMode:TTransparentMode;
    fTranspColor:TColor;
    procedure SetValue(v:integer);
    procedure SetBar(b:TBitmap);
    procedure SetSlider(b:TBitmap);
    procedure SetSliderWidth(w:integer);
    procedure SetTransp(b:boolean);
    procedure SetTranspMode(m:TTransparentMode);
    procedure SetTranspColor(c:TColor);
//    procedure CopyParentImage(B:TBitmap);
  protected
    procedure Paint; override;
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseMove(Shift:TShiftState; X,Y:Integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    property OnChange:TNotifyEvent read fOnChange write fOnChange;
    property OnShowValue:TNotifyEvent read fOnShow write fOnShow;
    property OnEndShowValue:TNotifyEvent read fOnEndShow write fOnEndShow;
    property Value: integer read fValue write SetValue;
    property BackgroundBitmap:TBitmap read fBar write SetBar;
    property SliderBitmap:TBitmap read fSlider write SetSlider;
    property SliderSize:integer read fSliderWidth write SetSliderWidth;
    property Transparent:boolean read fTransp write SetTransp;
    property TransparentMode:TTransparentMode read fTranspMode write SetTranspMode;
    property TransparentColor:TColor read fTranspColor write SetTranspColor;
    property Hint;
    property ShowHint;
    property Enabled;
  end;

procedure Register;

implementation
{==============================================================================}
constructor TxSlider.Create(AOwner:TComponent);
begin
 inherited;
 Width:=100; Height:=20;
 fBar:=TBitmap.Create; fSlider:=TBitmap.Create;
 fSliderWidth:=10; fShift:=-1;
 fValue:=500; fTransp:=False; fTranspMode:=tmAuto; fTranspColor:=clBlack;
end;
{==============================================================================}
destructor TxSlider.Destroy;
begin
 fBar.Free; fSlider.Free;
 inherited;
end;
{==============================================================================}
procedure TxSlider.Paint;
Var Pos:integer;
    fBitmap:TBitmap;
begin
 fBitmap:=TBitmap.Create; fBitmap.Width:=Width; fBitmap.Height:=Height;

 If (not fBar.Empty) and (not fSlider.Empty)
 Then Begin
  fBitmap.Canvas.CopyMode:=cmSrcCopy;
  If (not (csDesigning in ComponentState)) and (not (csLoading in ComponentState)) and fTransp
  Then CopyParentImage(Self,fBitmap.Canvas);
  fBitmap.Canvas.StretchDraw(Rect(0,0,Width,Height),fBar);
  Pos:=Round((Width-fSliderWidth)*(fValue/1000));
  fBitmap.Canvas.StretchDraw(Rect(Pos,0,Pos+fSliderWidth,Height),fSlider);
 End
 Else Begin
  fBitmap.Canvas.Brush.Color:=clBlack;
  fBitmap.Canvas.FrameRect(Rect(1,1,Width-1,Height-1));
 End;
 Canvas.CopyMode:=cmSrcCopy;
 Canvas.Draw(0,0,fBitmap);
 fBitmap.Free;
end;
{==============================================================================}
procedure TxSlider.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
Var Pos:integer;
begin
 If Button<>mbLeft Then exit;
 Pos:=Round((Width-fSliderWidth)*(fValue/1000));
 {On slider}
 If (X>=Pos) and (X<=Pos+fSliderWidth)
 Then Begin fShift:=X-Pos; Paint; If Assigned(fOnShow) Then fOnShow(Self); exit; End;

 {Out of slider}
 If X<=fSliderWidth Then Begin fValue:=0; fShift:=X; Paint; If Assigned(fOnShow) Then fOnShow(Self); exit; End;
 If X>=(Width-fSliderWidth) Then Begin fValue:=1000; fShift:=X-(Width-fSliderWidth); Paint; If Assigned(fOnShow) Then fOnShow(Self); exit; End;

 If (X>fSliderWidth) and (X<(Width-fSliderWidth))
 Then Begin fShift:=fSliderWidth div 2; Pos:=X-fShift;
 fValue:=Round(Pos/(Width-fSliderWidth)*1000); Paint; If Assigned(fOnShow) Then fOnShow(Self); End;
end;
{==============================================================================}
procedure TxSlider.MouseMove(Shift:TShiftState; X,Y:Integer);
var Pos:integer;
begin
 If not (ssLeft in Shift) Then exit; If fShift<0 Then exit;
 Pos:=X-fShift; If Pos<0 Then Pos:=0; If Pos>(Width-fSliderWidth) Then Pos:=(Width-fSliderWidth);
 fValue:=Round(Pos/(Width-fSliderWidth)*1000);
 Paint; If Assigned(fOnShow) Then fOnShow(Self);
end;
{==============================================================================}
procedure TxSlider.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
begin
 If Button<>mbLeft Then exit;
 fShift:=-1; If Assigned(fOnEndShow) Then fOnEndShow(Self);
 if Assigned(fOnChange) then fOnChange(Self);
end;
{==============================================================================}
procedure TxSlider.SetValue(v:integer);
begin
 If fShift>=0 Then exit;
 If v<0 Then v:=0; If v>1000 Then v:=1000; fValue:=v; Paint;
end;
{==============================================================================}
procedure TxSlider.SetSliderWidth(w:integer);
begin
 If w<3 Then exit; If w>=Width Then exit; fSliderWidth:=w; Paint;
end;
{==============================================================================}
procedure TxSlider.SetBar(b:TBitmap);
begin
 fBar.Assign(b); Paint;
end;
{==============================================================================}
procedure TxSlider.SetSlider(b:TBitmap);
begin
 fSlider.Assign(b); Paint;
end;
{==============================================================================}
procedure TxSlider.SetTransp(b:boolean);
begin
 fBar.Transparent:=b; fSlider.Transparent:=b; fTransp:=b; Paint;
end;
{==============================================================================}
procedure TxSlider.SetTranspMode(m:TTransparentMode);
begin
 fBar.TransparentMode:=m; fSlider.TransparentMode:=m; fTranspMode:=m; Paint;
end;
{==============================================================================}
procedure TxSlider.SetTranspColor(c:TColor);
begin
 If fTranspMode=tmAuto Then Begin fTranspColor:=c; exit; End;
 fBar.TransparentColor:=c; fSlider.TransparentColor:=c; Paint;
end;
{==============================================================================}
{procedure TxSlider.CopyParentImage(B:TBitmap);
var ParentCanvas:TCanvas;
begin
 ParentCanvas:=TForm(Parent).Canvas; B.Width:=Width; B.Height:=Height;
 B.Canvas.CopyMode:=cmSrcCopy;
 B.Canvas.CopyRect(Rect(0,0,Width,Height),ParentCanvas,BoundsRect);
end;
{==============================================================================}
procedure Register;
begin
 RegisterComponents('XRat', [TxSlider]);
end;
{==============================================================================}
end.
