unit xVolume;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Extctrls, xGraphLib;

type TOrientation=(orHorizontal,orVertical);

type
  TxVolume = class(TGraphicControl)
  private
    fBarLt,fBarDk,fSlider:TBitmap;
    fValue,fSliderWidth,fShift:integer;
    fOnChange,fOnShow,fOnEndShow:TNotifyEvent;
    fOrient:TOrientation;
    fTransp,MouseOverControl:boolean;
    fTranspMode:TTransparentMode;
    fTranspColor:TColor;
    procedure SetValue(v:integer);
    procedure SetOrient(orient:TOrientation);
    procedure SetBarLt(b:TBitmap);
    procedure SetBarDk(b:TBitmap);
    procedure SetSlider(b:TBitmap);
    procedure SetSliderWidth(w:integer);
    procedure SetTransp(b:boolean);
    procedure SetTranspMode(m:TTransparentMode);
    procedure SetTranspColor(c:TColor);
    procedure ChangeEvent;
  protected
    procedure Paint; override;
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseMove(Shift:TShiftState; X,Y:Integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure MouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    property OnChange:TNotifyEvent read fOnChange write fOnChange;
    property OnShowValue:TNotifyEvent read fOnShow write fOnShow;
    property OnEndShowValue:TNotifyEvent read fOnEndShow write fOnEndShow;
    property Value: integer read fValue write SetValue default 50;
    property BackgroundLightBitmap:TBitmap read fBarLt write SetBarLt;
    property BackgroundDarkBitmap:TBitmap read fBarDk write SetBarDk;
    property SliderBitmap:TBitmap read fSlider write SetSlider;
    property SliderSize:integer read fSliderWidth write SetSliderWidth;
    property Orientation: TOrientation read fOrient write SetOrient default orHorizontal;
    property Transparent:boolean read fTransp write SetTransp;
    property TransparentMode:TTransparentMode read fTranspMode write SetTranspMode;
    property TransparentColor:TColor read fTranspColor write SetTranspColor;
    property Hint;
    property ShowHint;
  end;

procedure Register;

implementation
{==============================================================================}
constructor TxVolume.Create(AOwner:TComponent);
begin
 inherited;
 Width:=100; Height:=20; fValue:=50; fSliderWidth:=10; fShift:=-1; MouseOverControl:=False;
 fOrient:=orHorizontal;
 fBarLt:=TBitmap.Create; fBarDk:=TBitmap.Create;  fSlider:=TBitmap.Create;
 fTransp:=False; fTranspMode:=tmAuto; fTranspColor:=clBlack;
end;
{==============================================================================}
destructor TxVolume.Destroy;
begin
 fBarLt.Free; fBarDk.Free; fSlider.Free;
 inherited;
end;
{==============================================================================}
procedure TxVolume.Paint;
var fBitmap,fBackLt,fBackDk:TBitmap;
    Pos:integer;
begin
 fBitmap:=TBitmap.Create; fBackLt:=TBitmap.Create; fBackDk:=TBitmap.Create;
 fBitmap.Width:=Width; fBitmap.Height:=Height; fBackLt.Width:=Width;
 fBackLt.Height:=Height; fBackDk.Width:=Width; fBackDk.Height:=Height;

 If (not fBarDk.Empty) and (not fBarLt.Empty) and (not fSlider.Empty)
 Then Begin
  fBitmap.Canvas.CopyMode:=cmSrcCopy; fBackLt.Canvas.CopyMode:=cmSrcCopy;
  fBackDk.Canvas.CopyMode:=cmSrcCopy;
  If (not (csDesigning in ComponentState)) and (not (csLoading in ComponentState)) and fTransp
  Then CopyParentImage(Self,fBitmap.Canvas);
  fBackLt.Canvas.StretchDraw(Rect(0,0,Width,Height),fBarLt);
  fBackDk.Canvas.StretchDraw(Rect(0,0,Width,Height),fBarDk);
  If fOrient=orHorizontal
  Then Begin
   fBitmap.Canvas.CopyRect(Rect(0,0,Round(Width*fValue/100),Height),fBackLt.Canvas,Rect(0,0,Round(Width*fValue/100),Height));
   fBitmap.Canvas.CopyRect(Rect(Round(Width*fValue/100),0,Width,Height),fBackDk.Canvas,Rect(Round(Width*fValue/100),0,Width,Height));
   Pos:=Round((Width-fSliderWidth)*(fValue/100));
   fBitmap.Canvas.StretchDraw(Rect(Pos,0,Pos+fSliderWidth,Height),fSlider);
  End
  Else Begin
   fBitmap.Canvas.CopyRect(Rect(0,0,Width,Height-Round(Height*fValue/100)),fBackDk.Canvas,Rect(0,0,Width,Height-Round(Height*fValue/100)));
   fBitmap.Canvas.CopyRect(Rect(0,Height-Round(Height*fValue/100),Width,Height),fBackLt.Canvas,Rect(0,Height-Round(Height*fValue/100),Width,Height));
   Pos:=Round((Height-fSliderWidth)*(fValue/100));
   fBitmap.Canvas.StretchDraw(Rect(0,Height-Pos-fSliderWidth,Width,Height-Pos),fSlider);
  End;
 End
 Else Begin
  fBitmap.Canvas.Brush.Color:=clBlack;
  fBitmap.Canvas.FrameRect(Rect(1,1,Width-1,Height-1));
 End;

 Canvas.CopyMode:=cmSrcCopy;
 Canvas.Draw(0,0,fBitmap);
 fBitmap.Free; fBackLt.Free; fBackDk.Free;
end;
{==============================================================================}
procedure TxVolume.ChangeEvent;
begin
 If Assigned(fOnChange) Then fOnChange(Self);
end;
{==============================================================================}
procedure TxVolume.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
Var Pos:integer;
begin
 If Button<>mbLeft Then exit;
 If fOrient=orHorizontal
 Then Begin
  Pos:=Round((Width-fSliderWidth)*(fValue/100));
  {On slider}
  If (X>=Pos) and (X<=Pos+fSliderWidth)
  Then Begin fShift:=X-Pos; Paint; exit; End;
  {Out of slider}
  If X<=fSliderWidth Then Begin fValue:=0; fShift:=X; Paint; ChangeEvent; exit; End;
  If X>=(Width-fSliderWidth) Then Begin fValue:=100; fShift:=X-(Width-fSliderWidth); Paint; ChangeEvent; exit; End;
  If (X>fSliderWidth) and (X<(Width-fSliderWidth))
  Then Begin fShift:=fSliderWidth div 2; Pos:=X-fShift;
  fValue:=Round(Pos/(Width-fSliderWidth)*100); Paint; ChangeEvent; End;
 End
 Else Begin
  Pos:=Round((Height-fSliderWidth)*(fValue/100));
  {On slider}
  If (Y>=Height-Pos-fSliderWidth) and (Y<=Height-Pos)
  Then Begin fShift:=Y-(Height-Pos-fSliderWidth); Paint; exit; End;
  {Out of slider}
  If Y<=fSliderWidth Then Begin fValue:=100; fShift:=Y; Paint; ChangeEvent; exit; End;
  If Y>=(Height-fSliderWidth) Then Begin fValue:=0; fShift:=Y-(Height-fSliderWidth); Paint; ChangeEvent; exit; End;
  If (Y>fSliderWidth) and (Y<(Height-fSliderWidth))
  Then Begin fShift:=fSliderWidth div 2; Pos:=Y-fShift;
  fValue:=100-Round(Pos/(Height-fSliderWidth)*100); Paint; ChangeEvent; End;
 End;
end;
{==============================================================================}
procedure TxVolume.MouseMove(Shift:TShiftState; X,Y:Integer);
Var Pos:integer;
begin
 If not (ssLeft in Shift) Then exit; If fShift<0 Then exit;
 If fOrient=orHorizontal
 Then Begin Pos:=X-fShift; fValue:=Round(Pos/(Width-fSliderWidth)*100); End
 Else Begin Pos:=Y-fShift; fValue:=100-Round(Pos/(Height-fSliderWidth)*100); End;
 If fValue<0 Then fValue:=0; If fValue>100 Then fValue:=100; Paint;
 ChangeEvent;
 If Assigned(fOnShow) Then fOnShow(Self);
end;
{==============================================================================}
procedure TxVolume.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
begin
 If Button<>mbLeft Then exit;
 fShift:=-1; //ChangeEvent;
 If not MouseOverControl Then If Assigned(fOnEndShow) Then fOnEndShow(Self);
end;
{==============================================================================}
procedure TxVolume.MouseEnter(var Message: TMessage);
begin
 MouseOverControl:=True;
 If Assigned(fOnShow) Then fOnShow(Self);
end;
{==============================================================================}
procedure TxVolume.MouseLeave(var Message: TMessage);
begin
 MouseOverControl:=False;
 If fShift>=0 Then exit;
 If Assigned(fOnEndShow) Then fOnEndShow(Self);
end;
{==============================================================================}
procedure TxVolume.SetValue(v:integer);
begin
 If v<0 Then v:=0; If v>100 Then v:=100;
 fValue:=v; Paint; If MouseOverControl Then If Assigned(fOnShow) Then fOnShow(Self);
end;
{==============================================================================}
procedure TxVolume.SetSliderWidth(w:integer);
begin
 If w<3 Then exit;
 If fOrient=orHorizontal
 Then Begin If w>=Width Then exit; End
 Else Begin If w>=Height Then exit; End;
 fSliderWidth:=w; Paint;
end;
{==============================================================================}
procedure TxVolume.SetBarLt(b:TBitmap);
begin
 fBarLt.Assign(b); Paint;
end;
{==============================================================================}
procedure TxVolume.SetBarDk(b:TBitmap);
begin
 fBarDk.Assign(b); Paint;
end;
{==============================================================================}
procedure TxVolume.SetSlider(b:TBitmap);
begin
 fSlider.Assign(b); Paint;
end;
{==============================================================================}
procedure TxVolume.SetOrient(orient:TOrientation);
begin
 fOrient:=Orient; Paint;
end;
{==============================================================================}
procedure TxVolume.SetTransp(b:boolean);
begin
 fBarDk.Transparent:=b; fBarLt.Transparent:=b;  fSlider.Transparent:=b; fTransp:=b; Paint;
end;
{==============================================================================}
procedure TxVolume.SetTranspMode(m:TTransparentMode);
begin
 fBarDk.TransparentMode:=m; fBarLt.TransparentMode:=m; fSlider.TransparentMode:=m; fTranspMode:=m; Paint;
end;
{==============================================================================}
procedure TxVolume.SetTranspColor(c:TColor);
begin
 If fTranspMode=tmAuto Then Begin fTranspColor:=c; exit; End;
 fBarDk.TransparentColor:=c; fBarLt.TransparentColor:=c; fSlider.TransparentColor:=c; Paint;
end;
{==============================================================================}
procedure Register;
begin
 RegisterComponents('XRat', [TxVolume]);
end;
{==============================================================================}
end.
