unit xCDplay;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, MMsystem,
     extctrls, xPlayerInfo;

type TxCDplay = class (TComponent)
     private
      fTimer:TTimer;
      MCIopened,fPaused:boolean;
      wDeviceId,fLen,fCnt,fPos,fTrackNo,fTracks:longint;
      fOnStop,fOnPlay,fOnPause,fOnEOF:TNotifyEvent;
      fOnTime:TxPlayerTimeEvent;
      fOnError:TxPlayerErrorEvent;
      fOnPos:TxPlayerPositionEvent;
      procedure OpenMCI;
      procedure CloseMCI;
      procedure PlayMCI(Pos:integer);
      procedure StopMCI;
      procedure PauseMCI;
      procedure ResumeMCI;
      function GetPlayState:integer;
      procedure OnTimer(Sender:TObject);
     protected
     public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
     published
      // ----------------------------------
      procedure Play(TrackNo:byte);
      procedure Stop;
      procedure Pause;
      procedure Seek(Position:integer);
      function GetTime:longint;
      // ----------------------------------
      property OnStop:TNotifyEvent read fOnStop write fOnStop;
      property OnPlay:TNotifyEvent read fOnPlay write fOnPlay;
      property OnPause:TNotifyEvent read fOnPause write fOnPause;
      property OnEOF:TNotifyEvent read fOnEOF write fOnEOF;
      property OnError:TxPlayerErrorEvent read fOnError write fOnError;
      property OnTime:TxPlayerTimeEvent read fOnTime write fOnTime;
      property OnPosition:TxPlayerPositionEvent read fOnPos write fOnPos;
     End;

implementation

{==============================================================================}
constructor TxCDplay.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);
 wDeviceID:=0; 
 MCIopened:=false; 
 fTimer:=TTimer.Create(Self); fTimer.Interval:=200; fTimer.Enabled:=False;
 fTimer.OnTimer:=OnTimer;
end;
{==============================================================================}
destructor TxCDplay.Destroy;
begin
 If MCIopened Then CloseMCI;
 fTimer.Free;
 inherited Destroy;
end;
{==============================================================================}
{                                MCI related                                   }
{==============================================================================}
procedure TxCDplay.OpenMCI;
var OpenParms: TMCI_Open_Parms;
    SetParms:  TMCI_Set_Parms;
    StatusParm: TMCI_Status_Parms;
begin
 MCIopened:=False;
 fTracks:=0;

 OpenParms.lpStrDeviceType:='cdaudio';
 If mciSendCommand(0,MCI_OPEN,MCI_OPEN_TYPE,Longint(@OpenParms))<>0
 Then exit;

 // get device id
 wDeviceId:=OpenParms.wDeviceID;

 // set time format to miliseconds
 SetParms.dwTimeFormat:=MCI_FORMAT_MILLISECONDS;
 mciSendCommand(wDeviceID,MCI_SET,MCI_SET_TIME_FORMAT,LongInt(@SetParms));

 // get number of tracks
 StatusParm.dwItem:=MCI_STATUS_NUMBER_OF_TRACKS;
 mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
 fTracks:=StatusParm.dwReturn;

 If (fTrackNo<1) or (fTrackNo>fTracks) Then Begin CloseMCI; exit; End;

 // get track length
 StatusParm.dwItem:=MCI_STATUS_LENGTH;
 StatusParm.dwTrack:=fTrackNo;
 mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT or MCI_TRACK,Longint(@StatusParm));
 fLen:=StatusParm.dwReturn div 1000;

 // set time format to track/min/sec/frame
 SetParms.dwTimeFormat:=MCI_FORMAT_TMSF;
 mciSendCommand(wDeviceID,MCI_SET,MCI_SET_TIME_FORMAT,LongInt(@SetParms));

 MCIopened:=True;
end;
{==============================================================================}
procedure TxCDplay.CloseMCI;
begin
 If GetPlayState=1 Then StopMCI;
 mciSendCommand(wDeviceID,MCI_CLOSE,MCI_WAIT,0); MCIopened:=False;
end;
{==============================================================================}
procedure TxCDplay.PlayMCI(Pos:integer);
var PlayParms: TMCI_Play_Parms;
    mm,ss,i:longint;
begin
 fCnt:=0; fPaused:=false;

 // play
 i:=Round((Pos/1000)*fLen);
 mm:=i div 60; ss:=i-mm*60;

 PlayParms.dwFrom:=MCI_MAKE_TMSF(fTrackNo,mm,ss,0);
 PlayParms.dwTo:=MCI_MAKE_TMSF(fTrackNo+1,0,0,0);

 If fTrackNo<>fTracks
 Then mciSendCommand(wDeviceID,MCI_PLAY,MCI_FROM or MCI_TO,LongInt(@PlayParms))
 Else mciSendCommand(wDeviceID,MCI_PLAY,MCI_FROM,LongInt(@PlayParms));
 fTimer.Enabled:=True;
end;
{==============================================================================}
procedure TxCDplay.StopMCI;
begin
 fTimer.Enabled:=False;
 mciSendCommand(wDeviceID,MCI_STOP,MCI_WAIT,0);
end;
{==============================================================================}
procedure TxCDplay.PauseMCI;
begin
 fTimer.Enabled:=False;
 mciSendCommand(wDeviceID,MCI_STOP,MCI_WAIT,0);
end;
{==============================================================================}
procedure TxCDplay.ResumeMCI;
var PlayParms: TMCI_Play_Parms;
begin
 PlayParms.dwTo:=MCI_MAKE_TMSF(fTrackNo+1,0,0,0);
 mciSendCommand(wDeviceID,MCI_PLAY,MCI_TO,LongInt(@PlayParms));
 fTimer.Enabled:=True;
end;
{==============================================================================}
function TxCDplay.GetPlayState:integer;
var StatusParm: TMCI_Status_Parms;
    st:longint;
begin
 // 0-stop 1-Play 2-pause 3-eof
 Result:=0;
 If not MCIopened Then exit;

 StatusParm.dwItem := MCI_STATUS_MODE;
 mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
 st:=StatusParm.dwReturn;
 Case st of
  MCI_MODE_NOT_READY: Result:=0;
  MCI_MODE_PAUSE    : Result:=2;
  MCI_MODE_PLAY     : Result:=1;
  MCI_MODE_STOP     : Result:=0;
  MCI_MODE_OPEN     : Result:=0;
  MCI_MODE_RECORD   : Result:=0;
  MCI_MODE_SEEK     : Result:=1;
 end;
end;
{==============================================================================}
procedure TxCDplay.OnTimer(Sender:TObject);
var StatusParm: TMCI_Status_Parms;
    pos,CurTrack:longint;
begin
 Inc(fCnt); If fCnt>5 Then fCnt:=1;
 If fCnt=1 Then // 1000 milisec
 Begin
 // get current track number
 StatusParm.dwItem:=MCI_STATUS_CURRENT_TRACK;
 mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
 CurTrack:=StatusParm.dwReturn;

 If CurTrack=fTrackNo Then
  Begin
   // get position
   StatusParm.dwItem := MCI_STATUS_POSITION;
   mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
   Pos:=MCI_TMSF_MINUTE(StatusParm.dwReturn)*60+MCI_TMSF_SECOND(StatusParm.dwReturn);
   // on_time and on_position events
   If Assigned(fOnTime) Then Begin fOnTime(Self,pos,fLen-pos) End;
   If Assigned(fOnPos) Then Begin If fLen>0 Then fOnPos(Self,Round(Pos/fLen*1000)) Else fOnPos(Self,0) End;
  End;
 End;

 If GetPlayState=0 // 200 milisec
// If CurTrack>fTrackNo // 200 milisec
 Then Begin
  fTimer.Enabled:=False;
  If Assigned(fOnEOF) Then fOnEOF(Self);
  StopMCI; CloseMCI;
 End;

end;
{==============================================================================}
{                              Play Stop Pause Seek                            }
{==============================================================================}
procedure TxCDplay.Play(TrackNo:byte);
begin
 fPos:=-1;
 If MCIopened Then CloseMCI;
 fTrackNo:=TrackNo;
 OpenMCI;
 If Assigned(fOnPlay) Then fOnPlay(Self);
 If not MCIopened Then Begin If Assigned(fOnEOF) Then fOnEOF(Self); exit; End;
 PlayMCI(0);
end;
{==============================================================================}
procedure TxCDplay.Stop;
begin
 If not MCIopened Then exit;
 StopMCI; CloseMCI;
 If Assigned(fOnStop) Then fOnStop(Self);
end;
{==============================================================================}
procedure TxCDplay.Pause;
begin
 If not MCIopened Then exit;

 Case GetPlayState of
 //If not fPaused
 1: Begin PauseMCI; fPaused:=True; If Assigned(fOnPause) Then fOnPause(Self); End;
// 0: Begin ResumeMCI;fPaused:=False; If Assigned(fOnPlay) Then fOnPlay(Self); End;
 0: Begin
     fPaused:=False;
     If fPos<0 Then Begin ResumeMCI End Else Begin PlayMCI(fPos); fPos:=-1; End;
     If Assigned(fOnPlay) Then fOnPlay(Self);
    End;
 End;
end;
{==============================================================================}
procedure TxCDplay.Seek(Position:integer);
var pos:longint;
begin
 If not MCIopened Then exit;
// If GetPlayState=0 Then If Assigned(fOnPlay) Then fOnPlay(Self); PlayMCI(Position);
 fPos:=Position; Pos:=Round((fPos/1000)*fLen);
 If Assigned(fOnTime) Then Begin fOnTime(Self,Pos,fLen-Pos) End;
 If GetPlayState=1 Then PlayMCI(Position);
end;
{==============================================================================}
function TxCDplay.GetTime:longint;
begin
 Result:=fLen;
end;
{==============================================================================}
end.
