unit xMciplay;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, MMsystem,
     extctrls, xPlayerInfo;

type TxDeviceType = (dtWave,dtMpeg,dtMidi);

type TxMciplay = class (TComponent)
     private
      fTimer:TTimer;
      MCIopened:boolean;
      wDeviceId,fLen,fCnt,fPos:longint;
      fOnStop,fOnPlay,fOnPause,fOnEOF:TNotifyEvent;
      fOnTime:TxPlayerTimeEvent;
      fOnError:TxPlayerErrorEvent;
      fOnPos:TxPlayerPositionEvent;
      procedure OpenMCI(Filename:string; DeviceType:TxDeviceType);
      procedure CloseMCI;
      procedure PlayMCI(Pos:integer);
      procedure StopMCI;
      procedure PauseMCI;
      procedure ResumeMCI;
      function GetPlayState:integer;
      procedure OnTimer(Sender:TObject);
     protected
     public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
     published
      // ----------------------------------
      procedure Play(Filename:string;DeviceType:TxDeviceType);
      procedure Stop;
      procedure Pause;
      procedure Seek(Position:integer);
      function GetTime:longint;
      // ----------------------------------
      property OnStop:TNotifyEvent read fOnStop write fOnStop;
      property OnPlay:TNotifyEvent read fOnPlay write fOnPlay;
      property OnPause:TNotifyEvent read fOnPause write fOnPause;
      property OnEOF:TNotifyEvent read fOnEOF write fOnEOF;
      property OnError:TxPlayerErrorEvent read fOnError write fOnError;
      property OnTime:TxPlayerTimeEvent read fOnTime write fOnTime;
      property OnPosition:TxPlayerPositionEvent read fOnPos write fOnPos;
     End;

implementation

{==============================================================================}
constructor TxMciplay.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);
 wDeviceID:=0;
 MCIopened:=false; 
 fTimer:=TTimer.Create(Self); fTimer.Interval:=200; fTimer.Enabled:=False;
 fTimer.OnTimer:=OnTimer;
end;
{==============================================================================}
destructor TxMciplay.Destroy;
begin
 If MCIopened Then CloseMCI;
 fTimer.Free;
 inherited Destroy;
end;
{==============================================================================}
{                                MCI related                                   }
{==============================================================================}
procedure TxMciplay.OpenMCI(Filename:string; DeviceType:TxDeviceType);
var OpenParms: TMCI_Open_Parms;
    SetParms:  TMCI_Set_Parms;
begin
 MCIopened:=False;

 // open MCI for wave/mpeg/midi
 OpenParms.lpstrElementName:=PChar(filename);
 Case DeviceType of
  dtWave: OpenParms.lpStrDeviceType:='waveaudio';
  dtMpeg: OpenParms.lpStrDeviceType:='mpegvideo';
  dtMidi: OpenParms.lpStrDeviceType:='sequencer';
 End;
 If mciSendCommand(0,MCI_OPEN,MCI_OPEN_TYPE or MCI_OPEN_ELEMENT,Longint(@OpenParms))<>0
 Then exit;

 // get device id
 wDeviceId:=OpenParms.wDeviceID;

 // set time format to miliseconds
 SetParms.dwTimeFormat:=MCI_FORMAT_MILLISECONDS;
 If mciSendCommand(wDeviceID,MCI_SET,MCI_SET_TIME_FORMAT,LongInt(@SetParms))<>0
 Then Begin mciSendCommand(wDeviceID,MCI_CLOSE,MCI_WAIT,0); exit; End;

 MCIopened:=True;
end;
{==============================================================================}
procedure TxMciplay.CloseMCI;
begin
 mciSendCommand(wDeviceID,MCI_CLOSE,MCI_WAIT,0); MCIopened:=False;
end;
{==============================================================================}
procedure TxMciplay.PlayMCI(Pos:integer);
var PlayParms: TMCI_Play_Parms;
    StatusParm: TMCI_Status_Parms;
begin
 fCnt:=0;

 // get total length
 StatusParm.dwItem := MCI_STATUS_LENGTH;
 mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
 fLen:=StatusParm.dwReturn;

 // play
 PlayParms.dwFrom:=Round((Pos/1000)*fLen); fLen:=Round(fLen/1000);
 mciSendCommand(wDeviceID,MCI_PLAY,MCI_FROM,LongInt(@PlayParms));
 fTimer.Enabled:=True;
end;
{==============================================================================}
procedure TxMciplay.StopMCI;
begin
 fTimer.Enabled:=False;
 mciSendCommand(wDeviceID,MCI_STOP,MCI_WAIT,0);
end;
{==============================================================================}
procedure TxMciplay.PauseMCI;
begin
 fTimer.Enabled:=False;
 mciSendCommand(wDeviceID,MCI_PAUSE,MCI_WAIT,0);
end;
{==============================================================================}
procedure TxMciplay.ResumeMCI;
begin
 mciSendCommand(wDeviceID,MCI_RESUME,MCI_WAIT,0);
 fTimer.Enabled:=True;
end;
{==============================================================================}
function TxMciplay.GetPlayState:integer;
var StatusParm: TMCI_Status_Parms;
    st:longint;
begin
 // 0-stop 1-Play 2-pause 3-eof
 Result:=0;
 If not MCIopened Then exit;

 StatusParm.dwItem := MCI_STATUS_MODE;
 mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
 st:=StatusParm.dwReturn;
 Case st of
  MCI_MODE_NOT_READY: Result:=0;
  MCI_MODE_PAUSE    : Result:=2;
  MCI_MODE_PLAY     : Result:=1;
  MCI_MODE_STOP     : Result:=0;
  MCI_MODE_OPEN     : Result:=0;
  MCI_MODE_RECORD   : Result:=0;
  MCI_MODE_SEEK     : Result:=1;
 end;
end;
{==============================================================================}
procedure TxMciplay.OnTimer(Sender:TObject);
var StatusParm: TMCI_Status_Parms;
    pos:longint;
begin
 Inc(fCnt); If fCnt>5 Then fCnt:=1;
 If fCnt=1 Then // 1000 milisec
 Begin
  // get position
  StatusParm.dwItem := MCI_STATUS_POSITION;
  mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
  Pos:=Round(StatusParm.dwReturn/1000);
  // on_time and on_position events
  If Assigned(fOnTime) Then Begin fOnTime(Self,Pos,fLen-Pos) End;
  If Assigned(fOnPos) Then Begin If fLen>0 Then fOnPos(Self,Round(Pos/fLen*1000)) Else fOnPos(Self,0); End;
 End;

 If GetPlayState=0 // 200 milisec
 Then Begin
  fTimer.Enabled:=False;
  If Assigned(fOnEOF) Then fOnEOF(Self);
  StopMCI; CloseMCI;
 End;
end;
{==============================================================================}
{                              Play Stop Pause Seek                            }
{==============================================================================}
procedure TxMciplay.Play(Filename:string;DeviceType:TxDeviceType);
begin
 fPos:=-1;
 If MCIopened Then CloseMCI;
 OpenMCI(Filename,DeviceType);
 If Assigned(fOnPlay) Then fOnPlay(Self);
 If not MCIopened Then Begin If Assigned(fOnEOF) Then fOnEOF(Self); exit; End;
 PlayMCI(0);
end;
{==============================================================================}
procedure TxMciplay.Stop;
begin
 If not MCIopened Then exit;
 StopMCI; CloseMCI;
 If Assigned(fOnStop) Then fOnStop(Self);
end;
{==============================================================================}
procedure TxMciplay.Pause;
begin
 If not MCIopened Then exit;
 Case GetPlayState of
 1: Begin PauseMCI; If Assigned(fOnPause) Then fOnPause(Self); End;
 2: Begin
     If fPos<0 Then Begin ResumeMCI End Else Begin PlayMCI(fPos); fPos:=-1; End;
     If Assigned(fOnPlay) Then fOnPlay(Self);
    End;
 End;
end;
{==============================================================================}
procedure TxMciplay.Seek(Position:integer);
var pos:longint;
begin
 If not MCIopened Then exit;
 fPos:=Position; Pos:=Round((fPos/1000)*fLen);
 If Assigned(fOnTime) Then Begin fOnTime(Self,Pos,fLen-Pos) End;
 If GetPlayState=1 Then PlayMCI(Position);
end;
{==============================================================================}
function TxMciplay.GetTime:longint;
begin
 Result:=fLen;
end;
{==============================================================================}
end.
