unit xMpegplay;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, XaudioPlayer,
     xPlayerInfo;

type TxMpegplay = class (TComponent)
     private
      fPlayer:TXaudioPlayer;
      fPong,fDuration:integer;
      fState:byte;
      fOutput:string;
      fOnStop,fOnPlay,fOnPause,fOnEOF:TNotifyEvent;
      fOnTime:TxPlayerTimeEvent;
      fOnError:TxPlayerErrorEvent;
      fOnPos:TxPlayerPositionEvent;
      procedure OnPing(Sender:TXaudioPlayer; Tag:Cardinal);
      procedure OnPlayerState(Sender:TXaudioPlayer; State:Integer);
      procedure OnNACK(Sender: TXaudioPlayer; Command: Byte; Code: SmallInt);
      procedure OnDuration(Sender: TXaudioPlayer; Duration: Cardinal);
      procedure OnTimecode(Sender: TXaudioPlayer; Hours: Byte; Minutes: Byte; Seconds: Byte; Fractions: Byte);
      procedure OnInputPosition(Sender: TXaudioPlayer; Offset: Cardinal; Range: Cardinal);
     protected
     public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
     published
      // ----------------------------------
      procedure Play(Filename:string);
      procedure Stop;
      procedure Pause;
      procedure Seek(Position:integer);
      // ----------------------------------
      procedure SetPriority(n:byte);
      procedure SetQuality(n:byte);
      procedure SetChannels(n:byte);
      function GetTime:longint;
      // ----------------------------------
      procedure SetEQ(EQ:PPlayerEq);
      property PlayerState:byte read fState;
      property OutputDevice:string read fOutput write fOutput;
      // ----------------------------------
      property OnStop:TNotifyEvent read fOnStop write fOnStop;
      property OnPlay:TNotifyEvent read fOnPlay write fOnPlay;
      property OnPause:TNotifyEvent read fOnPause write fOnPause;
      property OnEOF:TNotifyEvent read fOnEOF write fOnEOF;
      property OnError:TxPlayerErrorEvent read fOnError write fOnError;
      property OnTime:TxPlayerTimeEvent read fOnTime write fOnTime;
      property OnPosition:TxPlayerPositionEvent read fOnPos write fOnPos;
     End;

implementation

{==============================================================================}
constructor TxMpegplay.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);
 fPlayer:=TXaudioPlayer.Create(Self);
 fPong:=-1; // Ping pong counter for play
 fState:=0; fDuration:=0; 
 fOutput:='';

 // xaudio sets
 fPlayer.SetPlayerMode(7){7}; // OUTPUT_AUTO_CLOSE ON_STOP, ON_EOF, ON_PAUSE
 fPlayer.SetInputPositionRange(1000); // 1 - 1000
 fPlayer.Priority:=4; // Highest priority

 // event procedures
 fPlayer.OnNotifyPong:=OnPing;
 fPlayer.OnNotifyPlayerState:=OnPlayerState;
 fPlayer.OnNotifyNAck:=OnNACK;
 fPlayer.OnNotifyStreamDuration:=OnDuration;
 fPlayer.OnNotifyInputTimecode:=OnTimecode;
 fPlayer.OnNotifyInputPosition:=OnInputPosition;
end;
{==============================================================================}
destructor TxMpegplay.Destroy;
begin
 fPlayer.OutputReset;
 //fPlayer.Exit;
 inherited Destroy;
end;
{==============================================================================}
function MakePath(Dir,Filename:string):string;
begin
 Result:='';
 If (Trim(Dir)='') or (Trim(Filename)='') Then exit;
 If Dir[Length(Dir)]<>'\' Then Result:=Dir+'\' Else Result:=Dir;
 Result:=Result+ExtractFilename(Filename);
 Result:=Copy(Result,1,Length(Result)-Length(ExtractFileExt(Filename)));
 Result:=Result+'.wav';  
end;
{==============================================================================}
procedure TxMpegplay.Play(Filename:string);
//var fPing:integer;
begin
 If fPong<>-1 Then exit;
 //While fPong<>-1 Do Application.ProcessMessages;
 //If (fState=1) or (fState=2) Then fPlayer.Stop;
 {writeln('InputOpen - ',Filename);}
 fPlayer.InputOpen(Filename); 
 If Lowercase(Copy(fOutput,1,4))='wav:'
 Then fPlayer.OutputOpen(MakePath(fOutput,Filename))
 Else fPlayer.OutputOpen(fOutput);

 {writeln('Play');} fPlayer.Play;
 {writeln('Ping');} fPlayer.Ping({fPing}1); fPong:=0;
 While fPong<>1 Do Application.ProcessMessages;
 fPong:=-1; //writeln('End of play proc');
end;
{==============================================================================}
procedure TxMpegplay.Stop;
begin
 If (fState=1) or (fState=2)
 Then Begin
  fPlayer.Stop; while fState<>0 Do Application.ProcessMessages;
 End;
end;
{==============================================================================}
procedure TxMpegplay.Pause;
begin
 If fState=1 Then Begin fPlayer.Pause; End;
 If fState=2 Then fPlayer.Play;
end;
{==============================================================================}
procedure TxMpegplay.Seek(Position:integer);
var i:integer;
begin
 i:=Position;
 If i<0 Then i:=0; If i>1000 Then i:=1000;
 If (fState=1) or (fState=2) Then Begin fPlayer.InputSeek(i,1000); End;
end;
{==============================================================================}
{                              Equalizer property                              }
{==============================================================================}
procedure TxMpegplay.SetEQ(EQ:PPlayerEq);
var EQSet:XA_EqualizerInfo;
    i,j:integer;
begin
 If EQ=nil
 Then Begin fPlayer.SetCodecEqualizer(nil) End
 Else Begin
  For i:=0 To 31 Do
   Begin
    j:=0;
    If i=0 Then j:=Eq[1];
    If i=1 Then j:=Eq[2];
    If i=2 Then j:=Eq[3];
    If i=3 Then j:=Eq[4];
    If i=4 Then j:=Eq[5];
    If i=5 Then j:=Eq[6];
    If (i>5) and (i<8) Then j:=Eq[7];
    If (i>7) and (i<12) Then j:=Eq[8];
    If (i>11) and (i<20) Then j:=Eq[9];
    If i>19 Then j:=Eq[10];
    j:=j-50; j:=Round(j*1.25); // EQrange is 50!!! j:=Round(j*2.5*EqRange/100)
    EQSet.left[i]:=j; EQSet.right[i]:=j;
   End;
  fPlayer.SetCodecEqualizer(@EQSet);
 End;
end;
{==============================================================================}
{                              On messages events                              }
{==============================================================================}
procedure TxMpegplay.OnPing(Sender:TXaudioPlayer; Tag:Cardinal);
begin
 fPong:=Tag; //writeln('Pong');
end;
{==============================================================================}
procedure TxMpegplay.OnPlayerState(Sender:TXaudioPlayer; State:Integer);
var fPing:integer;
begin
 fState:=State;
 Case fState of
 0: Begin If Assigned(fOnStop) Then fOnStop(Self); End;
 1: Begin If Assigned(fOnPlay) Then fOnPlay(Self); End;
 2: Begin If Assigned(fOnPause) Then fOnPause(Self); End;
 3: Begin
     Randomize; fPing:=Random(32000);
     fPlayer.OutputDrain; fPlayer.Ping(fPing);
     While fPing<>fPong Do Application.ProcessMessages; fPong:=-1;
     If Assigned(fOnEOF) Then fOnEOF(Self);
    End;
 End;
end;
{==============================================================================}
procedure TxMpegplay.OnNACK(Sender: TXaudioPlayer; Command: Byte; Code: SmallInt);
begin
 Case Command of
//XA_MSG_COMMAND_INPUT_OPEN: 'Error open file ';
  XA_MSG_COMMAND_PLAY: If Assigned(fOnError) Then fOnError(Self,'Can not play - audio output busy');
 end;
end;
{==============================================================================}
procedure TxMpegplay.OnDuration(Sender: TXaudioPlayer; Duration: Cardinal);
begin
 fDuration:=Duration;
end;
{==============================================================================}
procedure TxMpegplay.OnTimecode(Sender:TXaudioPlayer; Hours:Byte; Minutes:Byte; Seconds:Byte; Fractions:Byte);
var i:Longint;
begin
 i:=hours*3600+minutes*60+seconds;
 If Assigned(fOnTime) Then fOnTime(Self,i,fDuration-i);
end;
{==============================================================================}
procedure TxMpegplay.OnInputPosition(Sender: TXaudioPlayer; Offset: Cardinal; Range: Cardinal);
begin
 If Assigned(fOnPos) Then fOnPos(Self,Offset);
end;
{==============================================================================}
procedure TxMpegplay.SetPriority(n:byte);
begin
 If (n<0) or (n>4) Then fPlayer.Priority:=4 Else fPlayer.Priority:=n;
end;
{==============================================================================}
procedure TxMpegplay.SetQuality(n:byte);
begin
 If (n<0) or (n>2) Then fPlayer.SetCodecQuality(0) Else fPlayer.SetCodecQuality(2-n);
end;
{==============================================================================}
procedure TxMpegplay.SetChannels(n:byte);
begin
 If (n<0) or (n>3) Then fPlayer.SetOutputChannels(0) Else fPlayer.SetOutputChannels(n);
end;
{==============================================================================}
function TxMpegplay.GetTime:longint;
begin
 result:=fDuration;
end;
{==============================================================================}
end.
