unit xPlayer;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, xMpegPlay,
     xMciplay, xCDplay, xGapplay, xPlayerInfo;

type TxPlayer = class (TComponent)
     private
      Filetype:TxPlayerFileType;
      fRemain,fStereo,fDisableFormatChecking,fMpegWinCodec:boolean;
      fGap,fState:byte;
      fPos,fTime:longint;
      fTimeEp,fTimeRm:longint;
      fInfo:string;
      fMpegPlayer:TxMpegplay;
      fMciPlayer:TxMciplay;
      fCDPlayer:TxCDplay;
      fGapPlayer:TxGapplay;
      fOnStop,fOnPlay,fOnPause,fOnEOF:TNotifyEvent;
      fOnTime:TxPlayerTimeEvent;
      fOnError:TxPlayerErrorEvent;
      fOnPos:TxPlayerPositionEvent;
      function GetTime:longint;
      function GetOutput:string;
      procedure SetOutput(s:string);
      procedure OnPlayEv(Sender:TObject);
      procedure OnPauseEv(Sender:TObject);
      procedure OnStopEv(Sender:TObject);
      procedure OnEofEv(Sender:TObject);
      procedure OnTimeEv(Sender: TObject; TimeEplased,TimeRemaining:Integer);
     protected
     public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
     published
      function GetFileInfo(Filename:string):TFileInfo;
      function GetFileMasks:string;
      // ----------------------------------
      function Play(Filename:string):TFileInfo;
      procedure Stop;
      procedure Pause;
      procedure Seek(Position:integer);
      // ----------------------------------
      procedure SetPriority(n:byte);
      procedure SetQuality(n:byte);
      procedure SetChannels(n:byte);
      // ----------------------------------
      procedure SetEQ(EQ:PPlayerEq);
      property Gap:byte read fGap write fGap;
      property PlayerState:byte read fState;
      property RemainingTime:boolean read fRemain write fRemain{SetRemain};
      property Stereo:boolean read fStereo;
      property FormatInfo:string read fInfo;
      property Time:longint read GetTime;
      property DisableFormatChecking:boolean read fDisableFormatChecking write fDisableFormatChecking;
      property UseMpegWindowsCodec:boolean read fMpegWinCodec write fMpegWinCodec; 
      property OutputDevice:string read GetOutput write SetOutput;
      // ----------------------------------
      property TimeEp:longint read fTimeEp write fTimeEp;
      property TimeRm:longint read fTimeRm write fTimeRm;
      // ----------------------------------
      property OnStop:TNotifyEvent read fOnStop write fOnStop;
      property OnPlay:TNotifyEvent read fOnPlay write fOnPlay;
      property OnPause:TNotifyEvent read fOnPause write fOnPause;
      property OnEOF:TNotifyEvent read fOnEOF write fOnEOF;
      property OnError:TxPlayerErrorEvent read fOnError write fOnError;
      property OnTime:TxPlayerTimeEvent read fOnTime write fOnTime;
      property OnPosition:TxPlayerPositionEvent read fOnPos write fOnPos;
     End;

//procedure Register;
procedure Delay(Msecs:longint);

implementation
{==============================================================================}
{procedure Register;
begin
  RegisterComponents('XRat', [TxPlayer]);
end;}
{==============================================================================}
constructor TxPlayer.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);
 fMpegPlayer:=TxMpegplay.Create(Self);
 fMciPlayer:=TxMciplay.Create(Self);
 fCDPlayer:=TxCDplay.Create(Self);
 fGapPlayer:=TxGapplay.Create(Self);
 filetype:=ftNONE; fGap:=0; fPos:=0; fInfo:=' ';
 fRemain:=False; fState:=0; fTimeEp:=0; fTimeRm:=0;
 fDisableFormatChecking:=False; fMpegWinCodec:=False;
end;
{==============================================================================}
destructor TxPlayer.Destroy;
begin
 FMpegPlayer.Free; fMciPlayer.Free; fCDPlayer.Free; fGapPlayer.Free;
 inherited Destroy;
end;
{==============================================================================}
{                            Play Stop Pause Seek OnEOF                        }
{==============================================================================}
function TxPlayer.Play(Filename:string):TFileInfo;
var A:TCdaInfo;
    B:TFileInfo;
begin
 If (fState=1) or (fState=2) Then Stop;
 fStereo:=True; fInfo:=' ';
 B:=GetFileInfo(Filename); fInfo:=B.Info; fStereo:=B.Stereo; fTime:=B.Song.Duration; 
 filetype:=B.FileType; Result:=B;

 If (filetype=ftMPEG) and fMpegWinCodec Then filetype:=ftWMPEG; 

 Case filetype of
 ftMPEG:Begin // ---------------------------------------------------------------
          fMpegPlayer.OnStop:=OnStopEv; fMpegPlayer.OnPlay:=OnPlayEv;
          fMpegPlayer.OnPause:=OnPauseEv; fMpegPlayer.OnEOF:=OnEofEv;
          fMpegPlayer.OnError:=fOnError; fMpegPlayer.OnTime:=OnTimeEv;
          fMpegPlayer.OnPosition:=fOnPos;
          fMpegPlayer.Play(Filename); 
        End;
 ftWAV,
 ftMIDI,
 ftWMPEG:Begin // ---------------------------------------------------------------
          fMciPlayer.OnStop:=OnStopEv; fMciPlayer.OnPlay:=OnPlayEv;
          fMciPlayer.OnPause:=OnPauseEv; fMciPlayer.OnEOF:=OnEofEv;
          fMciPlayer.OnError:=fOnError; fMciPlayer.OnTime:=OnTimeEv;
          fMciPlayer.OnPosition:=fOnPos;
          Case filetype of
           ftWAV:  Begin
                    fMciPlayer.Play(Filename,dtWave);
                   End;
           ftMIDI: Begin
                    fMciPlayer.Play(Filename,dtMidi);
                   End;
           ftWMPEG:Begin
                    fMciPlayer.Play(Filename,dtMpeg);
                   End;
          End;
        End;
 ftCD:  Begin // ---------------------------------------------------------------
          fCDPlayer.OnStop:=OnStopEv; fCDPlayer.OnPlay:=OnPlayEv;
          fCDPlayer.OnPause:=OnPauseEv; fCDPlayer.OnEOF:=OnEofEv;
          fCDPlayer.OnError:=fOnError; fCDPlayer.OnTime:=OnTimeEv;
          fCDPlayer.OnPosition:=fOnPos;
          A:=GetCDAinfo(filename);
          fCDPlayer.Play(A.TrackNo);
        End;
 ftNONE:Begin // ---------------------------------------------------------------
         If Assigned(fOnPlay) Then fOnPlay(Self);
         filetype:=ftGAP;
         fGapPlayer.OnStop:=OnStopEv;   fGapPlayer.OnPlay:=OnPlayEv;
         fGapPlayer.OnPause:=OnPauseEv; fGapPlayer.OnEOF:=OnEofEv;
         fGapPlayer.OnTime:=OnTimeEv;   fGapPlayer.OnPosition:=fOnPos;
         fGapPlayer.Play(0,0,0);
        End;  // ---------------------------------------------------------------
 End;
end;
{==============================================================================}
procedure TxPlayer.Stop;
begin
 Case filetype of
 ftMPEG:       fMpegPlayer.Stop;
 ftWAV,ftMIDI,ftWMPEG: fMciPlayer.Stop;
 ftCD:         fCDPlayer.Stop;
 ftGAP:        fGapPlayer.Stop;
 End;
end;
{==============================================================================}
procedure TxPlayer.Pause;
begin
 Case filetype of
 ftMPEG:        fMpegPlayer.Pause;
 ftWAV, ftMIDI, ftWMPEG: fMciPlayer.Pause;
 ftCD:          fCDPlayer.Pause;
 ftGAP:         fGapPlayer.Pause;
 End;
end;
{==============================================================================}
procedure TxPlayer.Seek(Position:integer);
begin
 Case filetype of
 ftMPEG:        fMpegPlayer.Seek(Position);
 ftWAV, ftMIDI, ftWMPEG: fMciPlayer.Seek(Position);
 ftCD:          fCDPlayer.Seek(Position);
 End;
end;
{==============================================================================}
{                          Play  Pause  Stop  Eof  Events                      }
{==============================================================================}
procedure TxPlayer.OnPlayEv(Sender:TObject);
begin
 If Assigned(fOnPlay) Then fOnPlay(Self);
 fState:=1;
end;
{==============================================================================}
procedure TxPlayer.OnPauseEv(Sender:TObject);
begin
 If Assigned(fOnPause) Then fOnPause(Self);
 fState:=2;
end;
{==============================================================================}
procedure TxPlayer.OnStopEv(Sender:TObject);
begin
 If Assigned(fOnStop) Then fOnStop(Self);
 fState:=0;
end;
{==============================================================================}
procedure TxPlayer.OnEofEv(Sender:TObject);
begin
 If filetype=ftGAP
 Then Begin
  fState:=3;
  If Assigned(fOnEOF) Then fOnEOF(Self);
  fInfo:=' ';
  exit;
 End;

 filetype:=ftGAP;
 fGapPlayer.OnStop:=OnStopEv;   fGapPlayer.OnPlay:=OnPlayEv;
 fGapPlayer.OnPause:=OnPauseEv; fGapPlayer.OnEOF:=OnEofEv;
 fGapPlayer.OnTime:=OnTimeEv;   fGapPlayer.OnPosition:=fOnPos;
 fGapPlayer.Play(fTimeEp,fTimeRm,fGap);
end;
{==============================================================================}
procedure TxPlayer.OnTimeEv(Sender: TObject; TimeEplased,TimeRemaining:Integer);
begin
 fTimeEp:=TimeEplased; fTimeRm:=TimeRemaining;
 If Assigned(fOnTime) Then fOnTime(Self,TimeEplased,TimeRemaining);
end;
{==============================================================================}
{                     Equalizer PlayerState RemainingTime                      }
{==============================================================================}
procedure TxPlayer.SetEQ(EQ:PPlayerEq);
begin
 fMpegPlayer.SetEQ(EQ);
end;
{==============================================================================}
{                            File Info and Masks                               }
{==============================================================================}
function TxPlayer.GetFileInfo(Filename:string):TFileInfo;
{------------------------------------------------------------------------------}
function GetFileName(Filename:string):string;
var s:string;
    i:integer;
begin
S:=ExtractFileName(Trim(Filename)); i:=Length(ExtractFileExt(S));
Result:=Copy(S,1,Length(S)-i);
end;
{------------------------------------------------------------------------------}
var Ext:string;
    ft:TxPlayerFileType;
    M:TMpegInfo;
    W:TWavInfo;
//  D:TMidInfo;
    C:TCdaInfo;
begin
 Result.Song.Duration:=0; Result.Song.Album:=''; Result.Song.Year:='';
 Result.Song.Selected:=False; Result.Song.LastSelected:=False;
 Result.Song.Active:=False; Result.Song.ShufflePos:=0;
 Result.Song.Volume:=0;

 Result.Song.Filename:=ExpandFilename(Trim(Filename));
 Result.Song.Title:=GetFileName(Filename);
 Result.Song.Artist:='#untagged#';
 Result.Stereo:=True; Result.DetailInfo:=''; Result.Info:=' '; Result.Song.Exists:=False;
 Result.FileType:=ftNONE;

 If not FileExists(Result.Song.Filename) Then exit;

 If fDisableFormatChecking
 Then Begin Result.FileType:=ftMPEG; Result.Song.Exists:=True; exit; End;

 ft:=ftNone;
 Ext:=Lowercase(ExtractFileExt(Filename));
 If (Ext='.mp1') or (Ext='.mp2') or (Ext='.mp3') Then ft:=ftMPEG;
 If Ext='.wav' Then ft:=ftWAV;
 If (Ext='.mid') or (Ext='.rmi') Then ft:=ftMIDI;
 If Ext='.cda' Then ft:=ftCD;

 M.Error:=True;

 If ft=ftWAV Then //------------------------------------------------------------
 Begin
  W:=GetWavInfo(Trim(Filename));
  If W.Error
  Then Begin
   M:=GetMpegInfo(Result.Song.Filename);
   If M.Error Then exit Else ft:=ftMPEG;
  End
  Else Begin
   Result.FileType:=ftWAV; Result.Song.Exists:=True;
   Result.DetailInfo:=W.Info;
   Result.Stereo:=(W.Channels>1); Result.Song.Duration:=W.Duration;
   Result.Song.Title:=ExtractFileName(Filename);
   Result.Info:=W.FormatCategory+' '+IntToStr(W.SampleRate div 1000)+' KHz '+ IntToStr(W.Bits)+' Bit';
   Result.Time:=W.FormatedDuration;
   exit;
  End;
 End;

 If ft=ftMPEG Then //-----------------------------------------------------------
 Begin
  If M.Error Then M:=GetMpegInfo(Trim(Filename)); If M.Error Then exit;
  Result.FileType:=ftMPEG; Result.Song.Exists:=True;
  Result.DetailInfo:=M.Info; Result.Time:=M.FormatedDuration;
  Result.Info:='MPEG '+IntToStr(M.Mpeg.BitRate)+' Kbps '+IntToStr(M.Mpeg.SampleRate div 1000)+' KHz';
  Result.Stereo:=M.Mpeg.Stereo; Result.Song.Duration:=M.Duration;
  Result.Song.Artist:='#untagged_mpeg#';
  If not M.Tag.Tagged Then exit;
  If Trim(M.Tag.Title)<>'' Then Begin Result.Song.Title:=M.Tag.Title; Result.Song.Artist:='Unknown'; End;
  If Trim(M.Tag.Artist)<>'' Then Result.Song.Artist:=M.Tag.Artist;
  If Trim(M.Tag.Album)<>'' Then Result.Song.Album:=M.Tag.Album;
  If Trim(M.Tag.Year)<>'' Then Result.Song.Year:=M.Tag.Year;
  exit;
 End;

 If ft=ftCD Then //---------------------------------------------------------------
 Begin
  C:=GetCdaInfo(Trim(Filename)); If C.Error Then exit;
  Result.FileType:=ftCD; Result.Song.Exists:=True;
  Result.DetailInfo:=C.Info; Result.Stereo:=True; Result.Song.Duration:=C.Duration;
  Result.Info:='CD Audio Track';
  Result.Song.Title:='CD Track '+IntToStr(C.TrackNo);
  Result.Time:=C.FormatedDuration;
  exit;
 End;

end;
{==============================================================================}
function TxPlayer.GetFileMasks:string;
begin
Result:='All supported audio & playlist files |*.mp1;*.mp2;*.mp3;*.wav;'{*.mid;*.rmi;}+'*.cda;*.m3u;*.pls|'+
       // 'All supported playlist files |*.m3u;*.pls|'+
        'Mpeg audio files (*.mp1;*.mp2;*.mp3)|*.mp1;*.mp2;*.mp3|'+
        'Wave audio files (*.wav)|*.wav|'+
       // 'MIDI audio files (*.mid;*.rmi)|*.mid;*.rmi|'+
        'CD audio tracks (*.cda)|*.cda|'+
        'PLS playlist files (*.pls)|*.pls|'+
        'M3U playlist files (*.m3u)|*.m3u|'+
        'All files (*.*)|*.*';
end;
{==============================================================================}
procedure Delay(MSecs:longint);
var
  FirstTickCount, Now: Longint;
begin
  FirstTickCount := GetTickCount;
  repeat
    Application.ProcessMessages;
    { allowing access to other controls, etc. }
    Now := GetTickCount;
  until (Now - FirstTickCount >= MSecs) or (Now < FirstTickCount);
end;
{==============================================================================}
procedure TxPlayer.SetPriority(n:byte);
begin
 fMpegPlayer.SetPriority(n);
end;
{==============================================================================}
procedure TxPlayer.SetQuality(n:byte);
begin
 fMpegPlayer.SetQuality(n);
end;
{==============================================================================}
procedure TxPlayer.SetChannels(n:byte);
begin
 fMpegPlayer.SetChannels(n);
end;
{==============================================================================}
function TxPlayer.GetTime:longint;
begin
 Result:=0;
 Case filetype of
 ftMPEG:        Result:=fMpegPlayer.GetTime;
 ftWAV, ftMIDI, ftWMPEG: Result:=fMCIPlayer.GetTime;
 ftCD:          Result:=fCDPlayer.GetTime;
 End;
end;
{==============================================================================}
function TxPlayer.GetOutput:string;
begin
 result:=fMpegPlayer.OutputDevice;
end;
{==============================================================================}
procedure TxPlayer.SetOutput(s:string);
begin
 fMpegPlayer.OutputDevice:=s;
end;
{==============================================================================}

end.
