unit xPlayerInfo;

interface

uses SysUtils, MMSystem;

{##############################################################################}
{#                                  COMMON                                    #}
{##############################################################################}

type TxPlayerTimeEvent = procedure(Sender: TObject; TimeEplased,TimeRemaining:Integer) of object;
type TxPlayerErrorEvent = procedure(Sender: TObject; ErrorMsg:string) of object;
type TxPlayerPositionEvent = procedure(Sender: TObject; Position:integer) of object;

type TPlayerEq = array[1..10] of byte;
type PPlayerEq = ^TPlayerEq;

type TxPlayerFileType = (ftMPEG, ftWMPEG, ftWAV, ftCD, ftMIDI, ftNONE, ftGAP);

type TSong = record
      Filename:string[255]; // Filename
      Duration:longint;     // Duration in seconds
      Title:string[{30}255];     // Title
      Artist:string[{30}255];    // Artist
      Album:string[{30}255];     // Album
      Year:string[4];       // Year
      Selected:boolean;     // Selected song
      LastSelected:boolean; // Last selected Item
      Active:boolean;       // Active (played) song
      ShufflePos:integer;   // Shuffle playing position
      Exists:boolean;       // File exists
      Volume:integer;
     End;

type TFileInfo = record
      Song:TSong;                // TSong structutre
      FileType:TxPlayerFileType; // File format
      DetailInfo:string;         // File info
      Info:string;               // Short file info
      Time:string;               // Formated time
      Stereo:boolean;            // Stereo / Mono
     End;

{##############################################################################}
{#                                  MPEG AUDIO                                #}
{##############################################################################}

const
   BUFFERSIZE={2047}4096; // Buffer size
   MAXFRAMESIZE=1060; // Max MP3 Frame Size - 320 Kbps
   MAXGENRES=124;
   GENRES: array [0..MaxGenres] of String = (
'Blues','Classic Rock','Country','Dance','Disco','Funk','Grunge','Hip-Hop','Jazz','Metal','New Age','Oldies',
'Other','Pop','R&B','Rap','Reggae','Rock','Techno','Industrial','Alternative','Ska','Death Metal','Pranks',
'Soundtrack','Euro-Techno','Ambient','Trip-Hop','Vocal','Jazz+Funk','Fusion','Trance','Classical','Instrumental',
'Acid','House','Game','Sound Clip','Gospel','Noise','AlternRock','Bass','Soul','Punk','Space','Meditative',
'Instrumental Pop','Instrumental Rock','Ethnic','Gothic','Darkwave','Techno-Industrial','Electronic','Pop-Folk',
'Eurodance','Dream','Southern Rock','Comedy','Cult','Gangsta','Top 40','Christian Rap','Pop/Funk','Jungle',
'Native American','Cabaret','New Wave','Psychadelic','Rave','Showtunes','Trailer','Lo-Fi','Tribal','Acid Punk',
'Acid Jazz','Polka','Retro','Musical','Rock & Roll','Hard Rock','Folk','Folk/Rock','National Folk','Swing','Bebob',
'Latin','Revival','Celtic','Bluegrass','Avantgarde','Gothic Rock','Progressive Rock','Psychedelic Rock','Symphonic Rock',
'Slow Rock','Big Band','Chorus','Easy Listening','Acoustic','Humour','Speech','Chanson','Opera','Chamber Music','Sonata',
'Symphony','Booty Bass','Primus','Porn Groove','Satire','Slow Jam','Club','Tango','Samba','Folklore',
'Ballad','Poweer Ballad','Rhytmic Soul','Freestyle','Duet','Punk Rock','Drum Solo','A Capela','Euro-House','Dance Hall');

type TFrameInfo = record
     Version:byte;          // 0,1,2,3 -> MPEG version 'Unknown', '1', '2', '2.5'
     Layer:byte;            // 0,1,2,3 -> Layer 'Unknown', '1', '2', '3'
     SampleRate:LongInt;    // Sample rate in Hz
     BitRate:LongInt;       // Bit Rate in Kbps
     ChannelMode:string;    // 'Stereo', 'Joint Stereo', 'Dual Channel Mono', 'Mono'
     Stereo:Boolean;        // true or false (stereo or mono)
     Padding:Boolean;       // Frame padding bit
     Copyright:Boolean;     // Copyright bit
     Original:Boolean;      // Original bit
     Privated:Boolean;      // Private bit
     CRC:boolean;           // Protected by CRC bit
     Emphasis:byte;         // 0,1,2,3 -> None, 50/15ms, Reserved, CCIT J.17
     FrameSize:Word;        // Frame size
     end;

type TTagInfo = record
     Tagged:boolean;             // Tag (tagged / not tagged)
     Title:string[30];           // Title
     Artist:string[30];          // Artist
     Album:string[30];           // Album
     Year:string[4];             // Year
     Comment:string[30];         // Comments
     Genre:byte;                 // Genre (id)
     GenreStr:string;            // Genre (string)
     end;

type TMpegInfo = record
     Error:boolean;              // Error (not mpeg format, file not founs etc.)
     Mpeg:TFrameInfo;            // Mpeg information
     Tag:TTagInfo;               // Tag information
     Info:string;                // For ex. 'MPEG 1.0 Layer 3, 128Kbps, 44 Khz, Stereo'
     FormatedDuration:string;    // Duration string (12:34 for example)
     Duration:longint;           // Duration (seconds)
     end;

{##############################################################################}
{#                                  WAVE AUDIO                                #}
{##############################################################################}

type TWavInfo = record
     Error:boolean;              // Error (not cda format, file not founs etc.)
     Info:string;                // 'PCM, 44100 Hz, 16 bit, Stereo' for example
     FormatCategory:string;      // 'PCM', 'ADPCM' etc...
     Channels:byte;              // Channels: 1-Mono 2-Stereo
     SampleRate:longint;         // Sample rate in Hz (44100 Hz for ex.)
     Bits:integer;               // Bits per sample: 8-16
     FormatedDuration:string;    // Duration string (12:34 for example)
     Duration:longint;           // Duration (seconds)
     end;

{##############################################################################}
{#                                   CD AUDIO                                 #}
{##############################################################################}

type TCdaInfo = record
     Error:boolean;              // Error (not cda format, file not founs etc.)
     TrackNo:integer;            // Track number
     ID:string;                  // CD identificator
     Info:string;                // 'Audio CD track 1' for example
     FormatedDuration:string;    // Duration string (12:34 for example)
     Duration:longint;           // Duration (seconds)
     end;

{##############################################################################}
{#                                     MIDI                                   #}
{##############################################################################}

type TMidInfo = record
     Error:boolean;              // Error (not midi format, file not founs etc.)
     Info:string;                // 'MIDI audio' for example
     FormatedDuration:string;    // Duration string (12:34 for example)
     Duration:longint;           // Duration (seconds)
     end;

{##############################################################################}
{#                     Functions & Procedures declaration                     #}
{##############################################################################}

function FormatTime(Sec:LongInt):string;
// MPEG
function GetMpegInfo(Filename:string):TMpegInfo;
function SaveTagID(Filename:string; TagID:TTagInfo):boolean;
function AnalyzeFrameHeader(H1,H2,H3:byte):TFrameInfo;
function AnalyzeTagID3(Buffer:Pointer):TTagInfo;
// WAVE
function GetWavInfo(Filename:string):TWavInfo;
// CD
function GetCdaInfo(Filename:string):TCdaInfo;
// MIDI
function GetMidInfo(Filename:string):TMidInfo;

implementation

{##############################################################################}
{#                                   COMMON                                   #}
{##############################################################################}
function FormatTime(Sec:LongInt):string;
var s:string;
begin
s:=IntToStr(Sec div 60); If Length(s)<2 Then s:='0'+s; Result:=s+':';
s:=IntToStr(Sec-(Sec div 60)*60); If Length(s)<2 Then s:='0'+s; Result:=Result+s;
end;
{==============================================================================}
function Spaces(i:integer):string;
var a:integer;
    s:string;
begin
 s:=''; for a:=1 to i do s:=s+' ';
 result:=s;
end;
{##############################################################################}
{#                                     MPEG                                   #}
{##############################################################################}
function CheckSYNC(H1,H2:byte):boolean;
begin
 If ((H1 and 255)=255) and ((H2 and 224)=224)
 Then Result:=True Else Result:=False;
end;
{==============================================================================}
function GetMpegInfo(Filename:string):TMpegInfo;
type TDataArray = array [0..BUFFERSIZE] of byte;
var F:File of byte;
    Buffer: ^TDataArray;
    fSize,FirstFramePos,i,j,k,n: LongInt;
    s:string;
    FrameInfo:TFrameInfo;
begin
// clear
Result.Error:=True; Result.Info:='Not supported format'; Result.FormatedDuration:='';
Result.Duration:=0;
Result.Mpeg.Version:=0; Result.Mpeg.Layer:=0; Result.Mpeg.SampleRate:=0;
Result.Mpeg.BitRate:=0; Result.Mpeg.ChannelMode:=''; Result.Mpeg.Stereo:=False;
Result.Mpeg.Padding:=False; Result.Mpeg.Copyright:=False; Result.Mpeg.Original:=False;
Result.Mpeg.CRC:=False; Result.Mpeg.Privated:=False;  Result.Mpeg.Emphasis:=0;  Result.Mpeg.FrameSize:=0;
Result.Tag.Tagged:=False; Result.Tag.Title:=''; Result.Tag.Artist:=''; Result.Tag.Album:='';
Result.Tag.Year:=''; Result.Tag.Comment:=''; Result.Tag.Genre:=12;
Result.Tag.GenreStr:='';

// open file
Try AssignFile(F,Filename); FileMode:=0; Reset(F) Except Exit End;

// get memory for buffer
Try New(Buffer) Except {$I-} CloseFile(F) {$I+}; exit End;

Try
  // get file size
  fSize:=FileSize(F);

  // if file too small then exit
  If fSize<BUFFERSIZE Then Exit;

  // read first block from file
  BlockRead(F,Buffer^,BUFFERSIZE+1);

  // Check for ID3v2
  If (Buffer^[0]=$49) and (Buffer^[1]=$44) and (Buffer^[2]=$33) and
     (Buffer^[3]<$FF) and (Buffer^[4]<$FF)
  Then Begin
   n:=Buffer^[9]+Buffer^[8]*128+Buffer^[7]*128*128+Buffer[6]*128*128*128+9;
   Seek(F,N); If Eof(F) Then exit;
   BlockRead(F,Buffer^,BUFFERSIZE+1); fSize:=fSize-N;
  End;

  {begin cycle}
  FirstFramePos:=-1;
  While True Do Begin

  // trying to find first frame
  i:=FirstFramePos;
  Repeat i:=i+1
  Until (i=BUFFERSIZE-MAXFRAMESIZE) or CheckSYNC(Buffer^[i],Buffer^[i+1]);
  If i=BUFFERSIZE-MAXFRAMESIZE Then exit;
  FirstFramePos:=i;

  // analyzing first frame header
  FrameInfo:=AnalyzeFrameHeader(Buffer^[i+1],Buffer^[i+2],Buffer^[i+3]);

  // check for valid version, layer, bitrate, samplerate and framesize
  If not ((FrameInfo.Version>0) and (FrameInfo.Layer>0) and (FrameInfo.SampleRate>0)
     and (FrameInfo.BitRate>0) and (FrameInfo.FrameSize>0)) Then Continue;

  // address of second frame header
  i:=FirstFramePos+FrameInfo.FrameSize;
  // checking second frame header
  If not CheckSYNC(Buffer^[i],Buffer^[i+1]) Then
  Begin
   i:=i-1; // maybe frame has one byte smaller length
   If not CheckSYNC(Buffer^[i],Buffer^[i+1]) Then Continue;
  End;

  // analyzing second frame header
  FrameInfo:=AnalyzeFrameHeader(Buffer^[i+1],Buffer^[i+2],Buffer^[i+3]);

  // check for valid version, layer, bitrate, samplerate and framesize
  If not ((FrameInfo.Version>0) and (FrameInfo.Layer>0) and (FrameInfo.SampleRate>0)
     and (FrameInfo.BitRate>0) and (FrameInfo.FrameSize>0)) Then Exit;

  Break;
  End; {end cycle}

  // fill result data
  Result.Mpeg:=FrameInfo;
  If FrameInfo.Version=3 Then s:='2.5' Else s:=IntToStr(FrameInfo.Version)+'.0';
  Result.Info:='MPEG '+s+' layer '+IntToStr(FrameInfo.Layer)+', '+IntToStr(FrameInfo.BitRate)+
  ' Kbps, '+IntToStr(FrameInfo.SampleRate div 1000)+' KHz, '+FrameInfo.ChannelMode;

  // calculate duration
  Result.Duration:=(fSize*8) div (FrameInfo.BitRate*1000);
  Result.FormatedDuration:=FormatTime(Result.Duration);

  // address of Xing's VBR tag
  i:=FirstFramePos+36;

  If (Buffer^[i]=Ord('X')) and (Buffer^[i+1]=Ord('i')) and
     (Buffer^[i+2]=Ord('n')) and (Buffer^[i+3]=Ord('g')) and (Buffer^[i+7]=15)
  Then Begin
   // total frames
   n:=Buffer^[i+8]; n:=n*256+Buffer^[i+9]; n:=n*256+Buffer^[i+10]; n:=n*256+Buffer^[i+11];
   // frame size
   j:=fSize div n;
   // bitrate
   k:=Trunc(j*FrameInfo.SampleRate/144000)-Integer(FrameInfo.Padding);
   // duration
   If k>0 Then Begin
    Result.Duration:=(fSize*8) div (k*1000);
    Result.FormatedDuration:=FormatTime(Result.Duration);
    Result.Mpeg.BitRate:=k;
    If FrameInfo.Version=3 Then s:='2.5' Else s:=IntToStr(FrameInfo.Version)+'.0';
    Result.Info:='MPEG '+s+' layer '+IntToStr(FrameInfo.Layer)+', '+IntToStr(k)+
    ' Kbps (VBR), '+IntToStr(FrameInfo.SampleRate div 1000)+' KHz, '+FrameInfo.ChannelMode;
   End;
  End;

  Result.Error:=False;

  // Read TAG
  Seek(F,FileSize(F)-128); BlockRead(F,Buffer^,128);
  Result.Tag:=AnalyzeTagID3(Buffer);

Finally
 {$I-}
 CloseFile(F);
 {$I+};
 FreeMem(Buffer);
End;
end;
{==============================================================================}
function AnalyzeTagID3(Buffer:Pointer):TTagInfo;
type TTagID = array [0..127] of char;
var pTagID:^TTagID;
    i:integer;
begin
Result.Tagged:=False; Result.Title:=''; Result.Artist:=''; Result.Album:='';
Result.Year:=''; Result.Comment:=''; Result.Genre:=12; Result.GenreStr:='';

pTagID:=Buffer;

// get TAG Info
If Copy(pTagID^,0,3)<>'TAG' Then exit;
For i:=0 To 127 Do If pTagID^[i]=#0 Then pTagID^[i]:=#32;
Result.Tagged:=True;
Result.Title:=TrimRight(Copy(pTagID^, 4, 30));
Result.Artist:=TrimRight(Copy(pTagID^, 34, 30));
Result.Album:=TrimRight(Copy(pTagID^, 64, 30));
Result.Year:=TrimRight(Copy(pTagID^, 94, 4));
Result.Comment:=TrimRight(Copy(pTagID^, 98, 30));
Result.Genre:=Ord(pTagID^[127]); If Result.Genre>MAXGENRES Then Result.Genre:=12;
Result.GenreStr:=Genres[Result.Genre];

end;
{==============================================================================}
function AnalyzeFrameHeader(H1,H2,H3:byte):TFrameInfo;
var b1,b2,b3,b4,b5:word;
begin
 Result.Version:=0; Result.Layer:=0; Result.SampleRate:=0;
 Result.BitRate:=0; Result.ChannelMode:=''; Result.Stereo:=False;
 Result.Padding:=False; Result.Copyright:=False; Result.Original:=False;
 Result.CRC:=False; Result.Emphasis:=0; Result.Privated:=False; Result.FrameSize:=0;

// get mpeg version
Case H1 and 24 of
 0:  Result.Version:=3; // Ver 2.5
 8:  Result.Version:=0; // Unknown
 16: Result.Version:=2; // Ver 2.0
 24: Result.Version:=1; // Ver 1.0
End;

// get mpeg layer
Case H1 and 6 of
 0: Result.Layer:=0; // Unknown
 2: Result.Layer:=3; // Layer 3
 4: Result.Layer:=2; // Layer 2
 6: Result.Layer:=1; // Layer 1
End;

// get mpeg bitrate index
b1:=0; b2:=0; b3:=0; b4:=0; b5:=0;
Case H2 and 240 of
  0: Begin b1:=0;   b2:=0;   b3:=0;   b4:=0;   b5:=0;   End;
 16: Begin b1:=32;  b2:=32;  b3:=32;  b4:=32;  b5:=8;   End;
 32: Begin b1:=64;  b2:=48;  b3:=40;  b4:=48;  b5:=16;  End;
 48: Begin b1:=96;  b2:=56;  b3:=48;  b4:=56;  b5:=24;  End;
 64: Begin b1:=128; b2:=64;  b3:=56;  b4:=64;  b5:=32;  End;
 80: Begin b1:=160; b2:=80;  b3:=64;  b4:=80;  b5:=40;  End;
 96: Begin b1:=192; b2:=96;  b3:=80;  b4:=96;  b5:=48;  End;
112: Begin b1:=224; b2:=112; b3:=96;  b4:=112; b5:=56;  End;
128: Begin b1:=256; b2:=128; b3:=112; b4:=128; b5:=64;  End;
144: Begin b1:=288; b2:=160; b3:=128; b4:=144; b5:=80;  End;
160: Begin b1:=320; b2:=192; b3:=160; b4:=160; b5:=96;  End;
176: Begin b1:=352; b2:=224; b3:=192; b4:=176; b5:=112; End;
192: Begin b1:=384; b2:=256; b3:=224; b4:=192; b5:=128; End;
208: Begin b1:=416; b2:=320; b3:=256; b4:=224; b5:=144; End;
224: Begin b1:=448; b2:=384; b3:=320; b4:=256; b5:=160; End;
240: Begin b1:=0;   b2:=0;   b3:=0;   b4:=0;   b5:=0;   End;
End;

If (Result.Version=1) and (Result.Layer=1) Then Result.BitRate:=b1;
If (Result.Version=1) and (Result.Layer=2) Then Result.BitRate:=b2;
If (Result.Version=1) and (Result.Layer=3) Then Result.BitRate:=b3;
If ((Result.Version=2) or (Result.Version=3)) and (Result.Layer=1) Then Result.BitRate:=b4;
If ((Result.Version=2) or (Result.Version=3)) and ((Result.Layer=2) or (Result.Layer=3)) Then Result.BitRate:=b5;

// get sample rate
b1:=0; b2:=0; b3:=0;
Case H2 and 12 of
 0:  Begin b1:=44100; b2:=22050; b3:=11025; End;
 4:  Begin b1:=48000; b2:=24000; b3:=12000; End;
 8:  Begin b1:=32000; b2:=16000; b3:=8000; End;
// 12: Begin b1:='reserved'; b2:='reserved'; b3:='reserved'; End;
End;

Case Result.Version of
 1: Result.SampleRate:=b1;
 2: Result.SampleRate:=b2;
 3: Result.SampleRate:=b3;
End;

// get padding
Result.Padding:=((H2 and 2)=2);

// get copyright
Result.Copyright:=((H3 and 8)=8);

// get original
Result.Original:=((H3 and 4)=4);

// get CRC
Result.CRC:=((H1 and 1)=1);

// get Private
Result.Privated:=((H2 and 2)=1);

// get Emphasis
Result.Emphasis:=(H3 and 3);

// get channel mode
Case H3 and 192 of
  0: Begin Result.ChannelMode:='Stereo'; Result.Stereo:=True; End;
 64: Begin Result.ChannelMode:='Joint stereo'; Result.Stereo:=True; End;
128: Begin Result.ChannelMode:='Dual channel mono'; Result.Stereo:=False; End;
192: Begin Result.ChannelMode:='Mono'; Result.Stereo:=False; End;
End;

// calculate frame size
If Result.SampleRate<=0 Then exit;

Try
If Result.Layer=1
Then Result.FrameSize:=Trunc(12*Result.BitRate*1000/Result.SampleRate+Integer(Result.Padding))*4
Else Result.FrameSize:=Trunc(144*Result.BitRate*1000/Result.SampleRate+Integer(Result.Padding));
Except
End;

end;
{==============================================================================}
function SaveTagID(Filename:string; TagID:TTagInfo):boolean;
type TTagArray = array [0..127] of char;
var TagData:^TTagArray;
    vTag:array [0..2] of char;
    s:string;
    F:File of char;
begin
Result:=False;

{create TAG ID string}
s:='TAG'+TagID.Title+Spaces(30-Length(TagID.Title))+
   TagID.Artist+Spaces(30-Length(TagID.Artist))+
   TagID.Album+Spaces(30-Length(TagID.Album))+
   TagID.Year+Spaces(4-Length(TagID.Year))+
   TagID.Comment+Spaces(30-Length(TagID.Comment))+ Chr(TagID.Genre);

{Reffer TagData pointer to TagID string}
TagData:=@s[1];

{write TAG ID to file}
Try AssignFile(F,Filename); FileMode:=2; Reset(F) Except exit End;

Try
 Seek(F,FileSize(F)-128);
 BlockRead(F,vTag,3);
 If vTag='TAG'
 Then Seek(F,FileSize(F)-128) // position for overwrite existing tag
 Else Seek(F,FileSize(F));    // position for append new tag
 BlockWrite(F,TagData^,128);
 Result:=True;
Finally
 {$I-}
 CloseFile(F);
 {$I+}
End;

end;
{##############################################################################}
{#                                     WAVE                                   #}
{##############################################################################}
function GetWavInfo(Filename:string):TWavInfo;
var F:File of byte;
    Buf:array [1..4] of byte;
    BytesPerSec,fSize:longint;
    fmtCategory:string;
begin
 // clear
 Result.Error:=True; Result.Info:=''; Result.Channels:=0; Result.SampleRate:=0;
 Result.Bits:=0; Result.FormatedDuration:=''; Result.Duration:=0;
 Result.FormatCategory:='';

 // open file
 Try AssignFile(F,Filename); FileMode:=0; Reset(F); fSize:=FileSize(F); Except Exit End;

 Try
 If fSize<24 Then exit;
 // check for WAV format
 Blockread(F,Buf,4); // check for 'RIFF'
 If (Chr(Buf[1])+Chr(Buf[2])+Chr(Buf[3])+Chr(Buf[4]))<>'RIFF' Then exit;

 Seek(F,8); Blockread(F,Buf,4); // check for 'WAVE'
 If (Chr(Buf[1])+Chr(Buf[2])+Chr(Buf[3])+Chr(Buf[4]))<>'WAVE' Then exit;

 Seek(F,20); Blockread(F,Buf,4); // check for version (only 1.0 supported - PCM)
// If (Buf[1]<>$1) or (Buf[2]<>$0) Then exit;
 fmtCategory:='';
 If Buf[1]+Buf[2]*256=1 Then Begin fmtCategory:='PCM'; Result.FormatCategory:='PCM' End;
 If Buf[1]+Buf[2]*256=2 Then Begin fmtCategory:='Microsoft ADPCM'; Result.FormatCategory:='ADPCM'; End;
 If Buf[1]+Buf[2]*256=7 Then Begin fmtCategory:='A/mu-Law Wave'; Result.FormatCategory:='MuLAW' End;
 If fmtCategory='' Then exit;

 // get channels number
 Result.Channels:=Buf[3];

 // get sample rate
 Blockread(F,Buf,4); Result.SampleRate:=Buf[1]+Buf[2]*256+Buf[3]*256*256+Buf[4]*256*256*256;;

 // get bites per sample
 Seek(F,34); Blockread(F,Buf,4); Result.Bits:=Buf[1]+Buf[2]*256;

 // get bytes per second
 BytesPerSec:=Round((Result.Channels*Result.SampleRate*Result.Bits)/8);

 // get duration
 Result.Duration:=Round((fSize-44)/BytesPerSec);
 {also you can use data chunk length instead of fSize-44, because it's more
  accurate way to get duration.}

 // formated duration
 Result.FormatedDuration:=FormatTime(Result.Duration);

 // info
 Result.Info:=fmtCategory+', '+IntToStr(Result.Samplerate)+' Hz, '+IntToStr(Result.Bits)+' bit, ';
 If Result.Channels=1
 Then Result.Info:=Result.Info+'Mono'
 Else Result.Info:=Result.Info+'Stereo';

 Result.Error:=False;
 Finally {$I-} CloseFile(F); {$I+}; End;
end;
{##############################################################################}
{#                                     CD                                     #}
{##############################################################################}
function GetCdaInfo(Filename:string):TCdaInfo;
var F:File of byte;
    Buf:array [1..4] of byte;
    fSize:longint;
begin
 // clear
 Result.Error:=True; Result.TrackNo:=0; Result.ID:=''; Result.FormatedDuration:='';
 Result.Duration:=0; Result.Info:='';

 // open file
 Try AssignFile(F,Filename); FileMode:=0; Reset(F);  fSize:=FileSize(F); Except Exit End;

 Try
 // check for CDA format
 If fSize<24 Then exit;
 Blockread(F,Buf,4); // check for 'RIFF'
 If (Chr(Buf[1])+Chr(Buf[2])+Chr(Buf[3])+Chr(Buf[4]))<>'RIFF' Then exit;

 Seek(F,8); Blockread(F,Buf,4); // check for 'CDDA'
 If (Chr(Buf[1])+Chr(Buf[2])+Chr(Buf[3])+Chr(Buf[4]))<>'CDDA' Then exit;

 Seek(F,20); Blockread(F,Buf,4); // check for CDA version (only 1.0 supported)
 If (Buf[1]<>$1) or (Buf[2]<>$0) Then exit;

 // get track number
 Result.TrackNo:=Buf[3]+Buf[4]*256;

 // get ID
 Blockread(F,Buf,4);
 Result.ID:=IntToHex(Buf[4],2)+IntToHex(Buf[3],2)+IntToHex(Buf[2],2)+IntToHex(Buf[1],2);

 // get duration
 Seek(F,40); Blockread(F,Buf,4);
 Result.FormatedDuration:=IntToStr(Buf[2]);
 If Length(Result.FormatedDuration)<2 Then Result.FormatedDuration:='0'+Result.FormatedDuration;
 Result.FormatedDuration:=IntToStr(Buf[3])+':'+Result.FormatedDuration;
 Result.Duration:=Buf[2]+Buf[3]*60;

 // get info
 Result.Info:='Audio CD, Serial Number: '+Result.ID+', Track Number: '+IntToStr(Result.TrackNo);

 Result.Error:=False;
 Finally {$I-} CloseFile(F); {$I+}; End;
end;
{##############################################################################}
{#                                   MIDI                                     #}
{##############################################################################}
function GetMidInfo(Filename:string):TMidInfo;
var OpenParms: TMCI_Open_Parms;
    SetParms:  TMCI_Set_Parms;
    StatusParm: TMCI_Status_Parms;
    wDeviceId: longint;
begin
 Result.Error:=True; Result.FormatedDuration:=''; Result.Duration:=0; Result.Info:='';

 // open MCI
 OpenParms.lpStrDeviceType:='sequencer';
 If mciSendCommand(0,MCI_OPEN,MCI_OPEN_TYPE or MCI_OPEN_ELEMENT,Longint(@OpenParms))<>0
 Then exit;

 // get device id
 wDeviceId:=OpenParms.wDeviceID;

 // set time format to miliseconds
 SetParms.dwTimeFormat:=MCI_FORMAT_MILLISECONDS;
 If mciSendCommand(wDeviceID,MCI_SET,MCI_SET_TIME_FORMAT,LongInt(@SetParms))<>0
 Then Begin mciSendCommand(wDeviceID,MCI_CLOSE,MCI_WAIT,0); exit; End;

 // get duration
 StatusParm.dwItem := MCI_STATUS_LENGTH;
 mciSendCommand(wDeviceID,MCI_STATUS,MCI_STATUS_ITEM or MCI_WAIT,Longint(@StatusParm));
 Result.Duration:=Round(StatusParm.dwReturn/1000);

 Result.FormatedDuration:=FormatTime(Result.Duration);
 Result.Info:='MIDI audio';
 Result.Error:=False;
end;
{##############################################################################}
{#                                                                            #}
{##############################################################################}
end.

