unit xPlaylist;
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls, Extctrls,
  xSongList, Dialogs, mxarrays, xPlayer, xPlayerInfo;

type TOnPlayEvent = procedure (ItemIndex:integer) of object;
type TOnChangeDuration = procedure (TotalDuration, SelDuration:string) of object;
type TxSortType = (byFilename, byTitle, byTime);
type TxPopupEvent = procedure (x,y:integer) of object;
type TxLoadPlaylist = procedure (PlaylistName:string) of object;
type TxChangeTitles = procedure (SongTitle,FormatInfo:string; Stereo:boolean) of object;

type
  TxPlaylist = class(TGraphicControl)
  private
    fPlayer:TxPlayer;
    SearchString:string;
    fShuffle,fAutoScroll,fShowSongNumbers,fAddAnyFiles:boolean;
    fOnChangeDuration:TOnChangeDuration;
    fList:TSongList;
    fFormatInfo,fFormat:string;
    fTextHeight:Cardinal;
    fPos,fKeySelectedShift,fKeySelectedPos:integer;
    fFont:TFont;
    ReadyToMove:byte;
    LastMovePos,DblClickPos,SearchPos:integer;
    fOnPopup:TxPopupEvent;
    fOnPlaylist:TxLoadPlaylist;
    fNColor,fAColor,fEColor:TColor;
    fOnChangeTitle:TxChangeTitles;
    fBeforePlay,fOnChangePos:TNotifyEvent;
    procedure SetNColor(A:TColor);
    procedure SetAColor(A:TColor);
    procedure SetEColor(A:TColor);
    // --------------------------
    {fOnStop,fOnPlay,fOnPause,fOnEOF:TNotifyEvent;
    fOnTime:TxPlayerTimeEvent;
    fOnError:TxPlayerErrorEvent;
    fOnPos:TxPlayerPositionEvent;}
    // --------------------------
    procedure SetTitleFormat(s:string);
    procedure SetFont(F:TFont);
    procedure SetShuffle(b:boolean);
    procedure SetTopPos(n:integer);
    procedure SetShowSongNumbers(b:boolean);
//    function PlaylistFileSupported(name:string):boolean;
  protected
    procedure Paint; override;
    procedure MouseLeave(var msg : tmessage); message CM_MOUSELEAVE;
    procedure MouseEnter(var msg : tmessage); message CM_MOUSEENTER;
    procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); override;
    procedure MouseMove(Shift:TShiftState; X,Y:Integer); override;
    procedure DblClick; override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    function GetFormatedString(ItemIndex:cardinal):string;
    function GetFormatedDuration(ItemIndex:cardinal):string;
    procedure GetPlaylistDuration(var TotalDuration, SelDuration:string);
    procedure RefreshDuration;
    property Songs:TSongList read fList write fList;
    property TopItem:integer read fPos write SetTopPos;
    function Rows:cardinal;
    property Shuffle:boolean read fShuffle write SetShuffle;
    property TitleFormat:string read fFormat write SetTitleFormat;
    property Font:TFont read fFont write SetFont;
    property NormalFontColor:TColor read fNColor write SetNColor;
    property ActiveFontColor:TColor read fAColor write SetAColor;
    property ErrorFontColor:TColor read fEColor write SetEColor;
    property FormatInfo:string read fFormatInfo;
    property OnChangeDuration:TOnChangeDuration read fOnChangeDuration write fOnChangeDuration;
    property OnPopupMenu:TxPopupEvent read fOnPopup write fOnPopup;
    property OnLoadPlayList:TxLoadPlaylist read fOnPlaylist write fOnPlaylist;
    property OnChangeTitles:TxChangeTitles read fOnChangeTitle write fOnChangeTitle;
    property OnChangePos:TNotifyEvent read fOnChangePos write fOnChangePos;
    property AutoScroll:boolean read fAutoScroll write fAutoScroll;
    property ShowSongNumbers:boolean read fShowSongNumbers write SetShowSongNumbers;
    // -----------------------
    function AddFile(name:string;addnotexist:boolean):integer;
    function AddDir(Path:string;Recursive:boolean):integer;
    function SaveList(Filename:string;Extendend:boolean):boolean;
    function LoadList(Filename:string;Extendend,ClearPlaylistBefore:boolean):integer;
    procedure Clear;
    procedure DeleteSelected;
    procedure Refresh(DeleteUnknown:boolean);
    // -----------------------
    procedure Search(Key:Char);
    procedure CancelSearch;
    // -----------------------
    procedure ScrollUp;
    procedure ScrollDown;
    procedure PageUp;
    procedure PageDown;
    procedure KeyHome;
    procedure KeyEnd;
    procedure KeyUp(Shift:TShiftState);
    procedure KeyDown(Shift:TShiftState);
    procedure KeyEnter;
    procedure KeyBackspace;
    // -----------------------
    property  Player:TxPlayer read fPlayer write fPlayer;
    procedure Play;
    procedure Stop;
    procedure Pause;
    procedure Seek(Position:integer);
    procedure Next;
    procedure Previous;
    function LastInList:boolean;
    // -----------------------
    property OnBeforePlay:TNotifyEvent read fBeforePlay write fBeforePlay;
    property AddAnyFilesToPlaylist:boolean read fAddAnyFiles write fAddAnyFiles;
    // -----------------------
    procedure Sort(SortType:TxSortType;Reverse:boolean);
    procedure RandomizeList;
    // -----------------------
    property Hint;
    property ShowHint;
  end;

procedure Register;

implementation

{$R xPlaylist.res}

{==============================================================================}
Function NormDir(Path:string):string;
Begin
 If Path[Length(Path)]='\' Then Result:=Path Else Result:=Path+'\';
End;
{==============================================================================}
{                        Object events (create,destroy,paint)                  }
{==============================================================================}
constructor TxPlaylist.Create(AOwner:TComponent);
begin
 inherited;
 Width:=100; Height:=100;

 fNColor:=clLime;
 fAColor:=clWhite;
 fEColor:=clTeal;

 Canvas.CopyMode:=cmSrcCopy;
 Canvas.Brush.Color:=clBlack;
 Canvas.Pen.Color:=clDkGray;
 Canvas.Font.Size:=8;
 Canvas.Font.Charset:=DEFAULT_CHARSET;
 Canvas.Font.Name:='Arial';
 Canvas.Font.Color:=fNColor;

 fFont:=TFont.Create;
 fFont.Size:=8; fFont.Charset:=DEFAULT_CHARSET; fFont.Name:='Arial';

 fList:=TSongList.Create;

 fPos:=0; fFormat:='%1 - %2'; ReadyToMove:=0;
 fKeySelectedShift:=0; fKeySelectedPos:=0;
 fShuffle:=False; SearchString:=''; SearchPos:=-1; fFormatInfo:=' ';
 fShowSongNumbers:=True; fAddAnyFiles:=False;

 fPlayer:=TxPlayer.Create(Self); fAutoScroll:=True;

end;
{==============================================================================}
destructor TxPlaylist.Destroy;
begin
 fList.Free; fPlayer.Free; fFont.Free;
 inherited;
end;
{==============================================================================}
procedure TxPlaylist.Paint;
{------------------------------------------------------------------------------}
procedure Say(Row,ItemIndex:integer);
var i:integer;
    s,ss:string;
begin
 If fList.Songs[ItemIndex].Selected Then Canvas.Brush.Color:=clBlue Else Canvas.Brush.Color:=clBlack;
 If fList.Songs[ItemIndex].Active
 Then Begin Canvas.Font.Color:=fAColor End
 Else Begin If fList.Songs[ItemIndex].Exists Then Canvas.Font.Color:=fNColor Else Canvas.Font.Color:=fEColor End;
 s:='  '+GetFormatedDuration(ItemIndex)+' '; i:=Canvas.TextWidth(S);
 Canvas.TextRect(Rect(0,Row*fTextHeight,Width{-fTextHeight}-i,Row*fTextHeight+fTextHeight),0,Row*fTextHeight,GetFormatedString(ItemIndex));
 If ItemIndex=SearchPos
 Then Begin
  Canvas.Font.Style:=fFont.Style+[fsUnderline];
  ss:=Copy(GetFormatedString(ItemIndex),Length(' '+IntToStr(ItemIndex+1)+'. ')+1,Length(SearchString));
  Canvas.TextOut(Canvas.TextWidth(' '+IntToStr(ItemIndex+1)+'. '),Row*fTextHeight,ss);
  Canvas.Font.Style:=fFont.Style;
 End;
 Canvas.TextRect(Rect(Width{-fTextHeight}-i,Row*fTextHeight,Width{-fTextHeight}-1,Row*fTextHeight+fTextHeight),Width{-fTextHeight}-i,Row*fTextHeight,s);
 Canvas.Brush.Color:=clBlack;
end;
{------------------------------------------------------------------------------}
procedure BlankSay(Row:integer);
begin
 Canvas.TextRect(Rect(0,Row*fTextHeight,Width{-fTextHeight},Row*fTextHeight+fTextHeight),0,Row*fTextHeight,' ');
end;
{------------------------------------------------------------------------------}
var i:integer;
begin
 Canvas.Brush.Color:=clBlack;
 fTextHeight:=Canvas.TextHeight('XRat Mpeg Player');
 // Check for resizing and fPos
 If (fPos>0) and ((fList.Count-fPos)<Rows) Then Begin fPos:=fList.Count-Rows; If fPos<0 Then fPos:=0 End;
 // Draw strokes
 For i:=0 To Rows Do If (fPos+i)<fList.Count Then Say(i,fPos+i) Else BlankSay(i);
 If Assigned(fOnChangePos) Then fOnChangePos(Self);
end;
{==============================================================================}
function TxPlaylist.Rows:cardinal;
begin
 Result:=Height div fTextHeight;
end;
{==============================================================================}
{                              Keyboard procedures                             }
{==============================================================================}
procedure TxPlaylist.ScrollUp;
begin
 If fList.Count=0 Then exit;
 If fPos>0 Then Begin fPos:=fPos-1; Paint; End;
 CancelSearch;
end;
{==============================================================================}
procedure TxPlaylist.ScrollDown;
begin
 If fList.Count=0 Then exit;
 If (fPos+Rows)<fList.Count Then Begin fPos:=fPos+1; Paint; End;
 CancelSearch;
end;
{==============================================================================}
procedure TxPlaylist.PageUp;
begin
 If fList.Count<Rows Then exit;
 fPos:=fPos-Rows+1; If fPos<0 Then fPos:=0; Paint;
 CancelSearch;
end;
{==============================================================================}
procedure TxPlaylist.PageDown;
begin
 If fList.Count<Rows Then exit;
 fPos:=fPos+Rows-1; If fPos>(fList.Count-Rows) Then fPos:=fList.Count-Rows; Paint;
 CancelSearch;
end;
{==============================================================================}
procedure TxPlaylist.KeyHome;
begin
 fPos:=0; CancelSearch;
end;
{==============================================================================}
procedure TxPlaylist.KeyEnd;
begin
 If (fList.Count>Rows) Then fPos:=fList.Count-Rows; CancelSearch;
end;
{==============================================================================}
procedure TxPlaylist.KeyEnter;
var i:integer;
begin
 i:=fList.GetLastSelected;
 If i>=0 Then Begin fList.SetActive(i); Paint; Play; End;
 CancelSearch;
end;
{==============================================================================}
procedure TxPlaylist.KeyUp(Shift:TShiftState);
var i:integer;
begin
 CancelSearch;
 If fList.Count=0 Then exit;
 i:=fList.GetLastSelected;
 If (i<fPos) or (i>fPos+Rows) Then
   Begin fList.UnselectAll; fList.Songs[fPos].Selected:=True; RefreshDuration;
   fList.SetLastSelected(fPos); Paint; exit End;

  If (ssAlt in Shift)
 Then Begin
  i:=i-1; If i<0 Then i:=0;
  fList.MoveSelected(mdUp);
 End
 Else Begin
  If not (ssShift in Shift) Then fList.UnselectAll;
//  If fKeySelectedShift=0 Then fKeySelectedPos:=i Else If fKeySelectedPos<>i Then fKeySelectedShift:=0;
//  fKeySelectedShift:=fKeySelectedShift-1;
//  fList.SelectRange(i,i+fKeySelectedShift);
  i:=i-1; If i<0 Then i:=0;
  fList.Songs[i].Selected:=True;
  fList.SetLastSelected(i);
  RefreshDuration;
 End;

 If i<fPos Then ScrollUp Else Paint;
end;
{==============================================================================}
procedure TxPlaylist.KeyDown(Shift:TShiftState);
var i:integer;
begin
 CancelSearch;
 If fList.Count=0 Then exit;
 i:=fList.GetLastSelected;
 If (i<fPos) or (i>fPos+Rows) Then
   Begin fList.UnselectAll; fList.Songs[fPos].Selected:=True; RefreshDuration;
   fList.SetLastSelected(fPos); Paint; exit End;

 If (ssAlt in Shift)
 Then Begin
  i:=i+1; If i>(fList.Count-1) Then i:=fList.Count-1;
  fList.MoveSelected(mdDown);
 End
 Else Begin
  If not (ssShift in Shift) Then fList.UnselectAll;
//  If fKeySelectedShift=0 Then fKeySelectedPos:=i Else If fKeySelectedPos<>i Then fKeySelectedShift:=0;
//  fKeySelectedShift:=fKeySelectedShift+1;
//  fList.SelectRange(i,i+fKeySelectedShift);
  i:=i+1; If i>(fList.Count-1) Then i:=fList.Count-1;
  fList.Songs[i].Selected:=True;
  fList.SetLastSelected(i);
  RefreshDuration;
 End;
 If i>(fPos+Rows-1) Then ScrollDown Else Paint;
end;
{==============================================================================}
procedure TxPlaylist.KeyBackspace;
var i:integer;
begin
 i:=fList.GetActive; If i<0 Then exit;
 fPos:=i-Rows div 2;
 If fPos<0 Then fPos:=0;
 If fPos>(fList.Count-1) Then fPos:=(fList.Count-1);
 Paint; 
end;
{==============================================================================}
{                                  Search                                      }
{==============================================================================}
procedure TxPlaylist.Search(Key:Char);
var s1,s2:string;
    i:integer;
begin
 If (Ord(Key)=27) or (Ord(key)=13) Then CancelSearch;
 s1:=SearchString+Key;
 For i:=0 To fList.Count-1 Do
 Begin
  s2:=' '+IntToStr(i+1)+'. '+s1;

  If fShowSongNumbers Then s2:=' '+IntToStr(i+1)+'. '+s1 Else s2:=' '+s1;
  If LowerCase(s2)=Copy(Lowercase(GetFormatedString(i)),1,Length(s2))
  Then Begin
   SearchString:=s1; SearchPos:=i; 
   fList.UnselectAll; fList.Songs[i].Selected:=True; fList.SetLastSelected(i);
   RefreshDuration; fPos:=i; Paint; Break;
  End;
 End;
end;
{==============================================================================}
procedure TxPlaylist.CancelSearch;
begin
 SearchString:=''; SearchPos:=-1; Paint;
end;
{==============================================================================}
{                               Mouse Events                                   }
{==============================================================================}
procedure TxPlaylist.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
var Row,Btn:integer;
begin
 CancelSearch;
 If fList.Count=0 Then exit; // patched 16.09.2000

 // Select & Unselect
 If X<Width-fTextHeight-1 Then
 Begin
  Row:=(Y div fTextHeight)+Integer((Y-(Y div fTextHeight)*fTextHeight)>0)-1;
  If Row<0 Then Row:=0; // patched 16.09.2000
  Btn:=0; If ssShift in Shift Then Btn:=1; If ssCtrl in Shift Then Btn:=2;
  Case Btn of
  0: Begin // None --------------------
      If Row+fPos<fList.Count
      Then Begin
       If not fList.Songs[Row+fPos].Selected
       Then Begin fList.UnselectAll; fList.Songs[Row+fPos].Selected:=True;
       fList.SetLastSelected(Row+fPos); ReadyToMove:=1; LastMovePos:=Row; End
       Else Begin ReadyToMove:=1; LastMovePos:=Row; End;
       RefreshDuration;
      End;
     End;
  1: Begin // Shift -------------------
      If Row+fPos<fList.Count // patched 16.09.2000
      Then Begin
       fList.UnselectAll;
       fList.SelectRange(fList.GetLastSelected,Row+fPos);
       RefreshDuration;
      End;
     End;
  2: Begin // Ctrl --------------------
      If Row+fPos<fList.Count Then Begin fList.Songs[Row+fPos].Selected:=not fList.Songs[Row+fPos].Selected; fList.SetLastSelected(Row+fPos); RefreshDuration; End;
     End;
  End;
  Paint;
 End;
end;
{==============================================================================}
procedure TxPlaylist.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
Var Row:integer;
begin
 If fList.Count=0 Then exit; // patched 16.09.2000

 // Select & Unselect after None click but not after move
 Row:=(Y div fTextHeight)+Integer((Y-(Y div fTextHeight)*fTextHeight)>0)-1;
 If Row<0 Then Row:=0; // patched 16.09.2000
 If (X<Width-fTextHeight-1) and (ReadyToMove=1) Then
 Begin
  ReadyToMove:=0;
  If not (LastMovePos=Row) Then exit;
  If Row+fPos<fList.Count Then If fList.Songs[Row+fPos].Selected
  Then Begin fList.UnselectAll; fList.Songs[Row+fPos].Selected:=True;
  fList.SetLastSelected(Row+fPos); LastMovePos:=Row; RefreshDuration; Paint; End
 End;
 ReadyToMove:=0;
 If (X<Width-fTextHeight-1) Then DblClickPos:=fPos+Row Else DblClickPos:=-1;

 If (Button=mbRight) and Assigned(fOnPopup) Then fOnPopup(x,y);
end;
{==============================================================================}
procedure TxPlaylist.MouseMove(Shift:TShiftState; X,Y:Integer);
Var Row,i:integer;
begin
 If fList.Count=0 Then exit; // patched 16.09.2000

 // Move selected items
 If (X<Width-fTextHeight-1) and (ReadyToMove>0) and (ssLeft in Shift) Then
 Begin
  Row:=(Y div fTextHeight)+Integer((Y-(Y div fTextHeight)*fTextHeight)>0)-1;
  If Row<0 Then Row:=0; // patched 16.09.2000
  For i:=1 To Abs(Row-LastMovePos) Do If (Row-LastMovePos)<0
  Then Begin fList.MoveSelected(mdUp); ReadyToMove:=2; Paint; End
  Else Begin fList.MoveSelected(mdDown); ReadyToMove:=2; Paint; End;
  While Row<1 Do Begin ScrollUp; Inc(Row); End;
  While Row>Rows-1 Do Begin ScrollDown; Dec(Row); End;
  LastMovePos:=Row;
 End;
end;
{==============================================================================}
procedure TxPlaylist.DblClick;
begin
 If fList.Count=0 Then exit;
 If (DblClickPos<fList.Count) and (DblClickPos>=0) Then
 Begin fList.SetActive(DblClickPos); Paint; Play; End;
end;
{==============================================================================}
procedure TxPlaylist.MouseLeave(var msg : tmessage);
begin

end;
{==============================================================================}
procedure TxPlaylist.MouseEnter(var msg : tmessage);
begin

end;
{==============================================================================}
{                     Add files, directory, clear, save and load list          }
{==============================================================================}
function TxPlaylist.AddFile(name:string;addnotexist:boolean):integer;
var A:TFileInfo;
begin
 Result:=-1;
 A:=fPlayer.GetFileInfo(name);
 If not fAddAnyFiles Then If (not A.Song.Exists) and (not addnotexist) Then exit;
 Result:=fList.Add(A.Song);
 //If Result=0 Then fList.Songs[0].Active:=True;
 RefreshDuration;
 SetShuffle(fShuffle);
 CancelSearch;
end;
{==============================================================================}
function TxPlaylist.AddDir(Path:string;Recursive:boolean):integer;
Var Dirs:TSearchRec;
    Res:integer;
Begin
Result:=-1;
// Make list file
Res:=FindFirst(NormDir(Path)+'*.*',faReadOnly+faHidden+faArchive+faDirectory,Dirs);
While Res=0 do
    Begin
     If ((Dirs.Attr and faDirectory) <> 0) and (Recursive)
     Then Begin
           If not ((Dirs.Name='.') or (Dirs.Name='..'))
           Then Begin
           If Result<0
            Then Result:=Adddir(NormDir(Path)+Dirs.Name,True)
            Else Adddir(NormDir(Path)+Dirs.Name,True);
           End
          End
     Else Begin
           If Result<0
           Then Result:=AddFile(NormDir(Path)+Dirs.Name,False)
           Else AddFile(NormDir(Path)+Dirs.Name,False);
          End;
     Res:=FindNext(Dirs);
    End;
FindClose(Dirs);
end;
{==============================================================================}
function TxPlaylist.SaveList(Filename:string;Extendend:boolean):boolean;
{------------------------------------------------------------------------------}
function CutDirFromFilename(Filename,Dir:string):string;
begin
 Result:=Filename;
 If Lowercase(Dir)=Lowercase(Copy(Filename,1,Length(Dir)))
 Then Result:=Copy(Filename,Length(Dir)+1,Length(Filename));
end;
{------------------------------------------------------------------------------}
var F:TextFile;
    i:integer;
    s,Dir:string;
begin
 Result:=False; Dir:=NormDir(ExtractFileDir(ExpandFileName(Filename)));
 AssignFile(F,Filename);
 Try Rewrite(F) Except exit End;
 Try
  If Extendend Then Try s:='[playlist]'; writeln(F,S) Except exit; End;
  For i:=0 To fList.Count-1 Do
  If Extendend Then Begin
   s:='File'+IntToStr(i+1)+'='+CutDirFromFilename(fList.Songs[i].Filename,Dir);
   Try writeln(F,S) Except exit; End;
   s:='Title'+IntToStr(i+1)+'='+fList.Songs[i].Title; Try writeln(F,S) Except exit; End;
   s:='Artist'+IntToStr(i+1)+'='+fList.Songs[i].Artist; Try writeln(F,S) Except exit; End;
   s:='Album'+IntToStr(i+1)+'='+fList.Songs[i].Album; Try writeln(F,S) Except exit; End;
   s:='Year'+IntToStr(i+1)+'='+fList.Songs[i].Year; Try writeln(F,S) Except exit; End;
   s:='Time'+IntToStr(i+1)+'='+IntToStr(fList.Songs[i].Duration); Try writeln(F,S) Except exit; End;
  End
  Else Begin
   Try writeln(F,CutDirFromFilename(fList.Songs[i].Filename,Dir)) Except exit; End;
  End;
 If Extendend Then
 Try s:='NumberOfEntries='+IntToStr(fList.Count); writeln(F,S); Except exit End;
 Result:=True;
 If Assigned(fOnPlaylist) Then fOnPlaylist(Trim(Filename));
 Finally
 {$I-} CloseFile(F); {$I+}
 End;
end;
{==============================================================================}
function TxPlaylist.LoadList(Filename:string;Extendend,ClearPlaylistBefore:boolean):integer;
{------------------------------------------------------------------------------}
function DecodeHead(Src:string; var Dst:string; var Pos:integer):integer;
var s:string;
    i,j:integer;
begin // result type 0-none 1-file 2-title 3-artist 4-album 5-year 6-time
 Result:=0; s:=trim(lowercase(src)); dst:=''; pos:=0; i:=0;
 If Copy(s,1,4)='file' Then Begin result:=1; i:=5; End;
 If Copy(s,1,5)='title' Then Begin result:=2; i:=6; End;
 If Copy(s,1,6)='artist' Then Begin result:=3; i:=7; End;
 If Copy(s,1,5)='album' Then Begin result:=4; i:=6; End;
 If Copy(s,1,4)='year' Then Begin result:=5; i:=5; End;
 If Copy(s,1,4)='time' Then Begin result:=6; i:=5; End;
 If Result=0 Then exit;
 j:=System.Pos('=',s); If j=0 Then Begin Result:=0; exit; End;
 Pos:=StrToIntDef(Copy(s,i,j-i),0); If Pos<1 Then Begin Result:=0; exit; End;
 Dst:=Copy(Trim(Src),j+1,Length(Trim(Src)));
end;
{------------------------------------------------------------------------------}
var F:TextFile;
    Buf:TStringlist;
    eList:TSongList;
    A:TSong;
    s,ss,p:string;
    Total,i,j,k:integer;
    PLSflag:boolean;
begin
If ClearPlaylistBefore Then Begin fList.Clear; fPos:=0; End;
Result:=-2;
If Extendend
Then Begin //===================================================================
 AssignFile(F,Filename); FileMode:=0;
 Try Reset(F) Except Result:=-1;  RefreshDuration; Paint; exit; End;
 Buf:=TStringList.Create; eList:=TSongList.Create; Total:=0; PLSflag:=False;
 Try
 While not Eof(F) Do
 Try
  Readln(F,S); S:=Trim(S); Buf.Add(S); s:=LowerCase(s);
  If s='[playlist]' Then PLSflag:=true;
  If Copy(s,1,16)='numberofentries=' Then Total:=StrToIntDef(Copy(s,17,length(s)),0);
 Except Result:=-1; Break; End;
 {$I-} CloseFile(F); {$I+}
 If (Result=-1) or (not PLSflag) or (Total=0) Then exit;

 A.Filename:=''; A.Duration:=0; A.Title:=''; A.Artist:=''; A.Album:=''; A.Year:='';
 A.Selected:=False; A.LastSelected:=False; A.Active:=False; A.ShufflePos:=0;
 A.Exists:=False; A.Volume:=0;

 For i:=1 To Total Do eList.Add(A);

 For i:=0 To Buf.Count-1 Do
 Begin
  j:=DecodeHead(Buf.Strings[i],ss,k);
  If (k-1)<eList.Count Then
  Case j of
  1: eList.Songs[k-1].Filename:=ss;
  2: eList.Songs[k-1].Title:=ss;
  3: eList.Songs[k-1].Artist:=ss;
  4: eList.Songs[k-1].Album:=ss;
  5: eList.Songs[k-1].Year:=ss;
  6: eList.Songs[k-1].Duration:=StrToIntDef(ss,0);
  End;
 End;

 P:=GetCurrentDir; SetCurrentDir(ExtractFileDir(Filename));
 For i:=0 To eList.Count-1 Do
 Begin
  A:=eList.Songs[i]^; If Trim(A.Filename)='' Then Continue;
  If Trim(A.Title)='' Then
  Begin
   s:=ExtractFileName(Trim(A.Filename));
   A.Title:=Copy(S,1,Length(S)-Length(ExtractFileExt(S)));
  End;
  If Trim(A.Artist)='' Then A.Artist:='Unknown'{'#untagged_mpeg#'};
  A.Filename:=ExpandFileName(A.Filename);
  If FileExists(A.Filename) Then A.Exists:=True Else A.Exists:=False;
  If Result<0 Then Result:=fList.Add(A) Else fList.Add(A);
 End;
 SetCurrentDir(p);
 //If ClearPlaylistBefore Then fList.SetActive(0);
 Finally
  Buf.Free; eList.Free; SetShuffle(Shuffle);
 End;
End
Else Begin //===================================================================
 AssignFile(F,Filename); FileMode:=0;
 P:=GetCurrentDir; SetCurrentDir(ExtractFileDir(Filename));
 Try Reset(F) Except Result:=-1; exit; End;
 While not Eof(F) Do
 Try
  Readln(F,S); If S[1]='#' Then Continue;
  If Result<0 Then Result:=Addfile(S,True) Else Addfile(S,True);
 Except Result:=-1; Break; End;
 {$I-} CloseFile(F); {$I+}
 SetCurrentDir(p);
End;
 If Assigned(fOnPlaylist) Then fOnPlaylist(Trim(Filename));
 RefreshDuration; Paint;
end;
{==============================================================================}
procedure TxPlaylist.Clear;
begin
 fList.Clear; RefreshDuration; fPos:=0; Paint;
 If Assigned(fOnPlaylist) Then fOnPlaylist(' ');
end;
{==============================================================================}
procedure TxPlaylist.DeleteSelected;
var i:integer;
begin
 i:=fList.Count-1;
 While i>=0 Do
 Begin
  If fList.Songs[i].Selected
  Then Begin
   If fList.Songs[i].Active
   Then Begin fList.Delete(i); If i>0 Then fList.SetActive(i-1); End
   Else Begin fList.Delete(i); End;
  End;
  Dec(i);
 End;
 SetShuffle(fShuffle); Paint; RefreshDuration;
 If Assigned(fOnPlaylist) and (fList.Count=0) Then fOnPlaylist(' ');
// Paint;
end;
{==============================================================================}
procedure TxPlaylist.Refresh(DeleteUnknown:boolean);
var i:integer;
    A:TFileInfo;
begin
 For i:=0 To fList.Count-1 Do
 Begin
  A:=fPlayer.GetFileInfo(fList.Songs[i].Filename);
  If (not A.Song.Exists)
  Then Begin
   If DeleteUnknown Then fList.Songs[i].Filename:='';
   fList.Songs[i].Exists:=False;
  End
  Else Begin
  fList.Songs[i].Duration:=A.Song.Duration; fList.Songs[i].Title:=A.Song.Title;
  fList.Songs[i].Artist:=A.Song.Artist; fList.Songs[i].Album:=A.Song.Album;
  fList.Songs[i].Year:=A.Song.Year; fList.Songs[i].Exists:=True;
  End;
 End;
 i:=0;
 If DeleteUnknown Then
 While i<fList.Count Do If fList.Songs[i].Filename='' Then fList.Delete(i) Else inc(i);
 RefreshDuration; Paint;
 If Assigned(fOnPlaylist) and (fList.Count=0) Then fOnPlaylist(' ');
end;
{==============================================================================}
{                                 Play Next Previous                           }
{==============================================================================}
procedure TxPlaylist.Play;
var i:integer;
    A:TFileInfo;
begin
 If fList.Count=0 Then exit;
 i:=fList.GetActive; If i=-1 Then Begin i:=0; fList.SetActive(0); Paint; End;
 If Assigned(fBeforePlay) Then fBeforePlay(Self);
 A:=fPlayer.Play(fList.Songs[i].Filename);

 If fList.Count=0 Then exit;

 If Assigned(fOnChangeTitle) Then fOnChangeTitle(GetFormatedString(i)+
 ' ('+GetFormatedDuration(i)+')',fPlayer.FormatInfo,fPlayer.Stereo);
 fFormatInfo:=fPlayer.FormatInfo;

 If A.Song.Exists
 Then Begin
  fList.Songs[i].Duration:=A.Song.Duration; fList.Songs[i].Title:=A.Song.Title;
  fList.Songs[i].Artist:=A.Song.Artist; fList.Songs[i].Album:=A.Song.Album;
  fList.Songs[i].Year:=A.Song.Year; fList.Songs[i].Exists:=True;
 End
 Else Begin fList.Songs[i].Exists:=False; End;

 If fAutoScroll
 Then Begin
  If i<fPos Then fPos:=i; If i>fPos+Rows-1 Then fPos:=i-Rows+1;
  If fPos<0 Then fPos:=0; If fPos>fList.Count-1 Then fPos:=fList.Count-1;
 End;
 Paint;
end;
{==============================================================================}
procedure TxPlaylist.Stop;
begin
 fPlayer.Stop;
end;
{==============================================================================}
procedure TxPlaylist.Pause;
begin
 fPlayer.Pause;
end;
{==============================================================================}
procedure TxPlaylist.Seek(Position:integer);
begin
 fPlayer.Seek(Position);
end;
{==============================================================================}
procedure TxPlaylist.Next;
var i,j,k:integer;
    B:TFileInfo;
begin
 If fList.Count=0 Then exit;
 i:=fList.GetActive; //If i=-1 Then i:=0;

 If fShuffle
 Then Begin
  j:=fList.Songs[i].ShufflePos;
  Inc(j); If j>fList.Count-1 Then j:=0;
  For k:=0 To fList.Count-1 Do If fList.Songs[k].ShufflePos=j Then i:=k;
 End
 Else Begin
  Inc(i);
  If fList.Count=0 Then exit;
  If i>fList.Count-1 Then i:=0;
 End;

 fList.SetActive(i);
 If fAutoScroll
 Then Begin
  If i<fPos Then fPos:=i; If i>fPos+Rows-1 Then fPos:=i-Rows+1;
  If fPos<0 Then fPos:=0; If fPos>fList.Count-1 Then fPos:=fList.Count-1;
 End;

 Paint;

 B:=fPlayer.GetFileInfo(fList.Songs[i].Filename);
 If Assigned(fOnChangeTitle) Then fOnChangeTitle(GetFormatedString(i)+
 ' ('+GetFormatedDuration(i)+')',B.Info,B.Stereo);
 fFormatInfo:=B.Info;
end;
{==============================================================================}
procedure TxPlaylist.Previous;
var i,j,k:integer;
    B:TFileInfo;
begin
 If fList.Count=0 Then exit;
 i:=fList.GetActive; //If i=-1 Then i:=0;

 If fShuffle
 Then Begin
  j:=fList.Songs[i].ShufflePos;
  Dec(j); If j<0 Then j:=fList.Count-1;
  For k:=0 To fList.Count-1 Do If fList.Songs[k].ShufflePos=j Then i:=k;
 End
 Else Begin
  Dec(i); If i<0 Then i:=fList.Count-1;
 End;

 fList.SetActive(i);

 If fAutoScroll
 Then Begin
  If i<fPos Then fPos:=i; If i>fPos+Rows-1 Then fPos:=i-Rows+1;
  If fPos<0 Then fPos:=0; If fPos>fList.Count-1 Then fPos:=fList.Count-1;
 End;

 Paint;

 B:=fPlayer.GetFileInfo(fList.Songs[i].Filename);
 If Assigned(fOnChangeTitle) Then fOnChangeTitle(GetFormatedString(i)+
 ' ('+GetFormatedDuration(i)+')',B.Info,B.Stereo);
 fFormatInfo:=B.Info;
end;
{==============================================================================}
function TxPlaylist.LastInList:boolean;
var i:integer;
begin
 Result:=False;
 If fList.Count=0 Then Begin Result:=True; exit; End; //If fShuffle Then exit;
 i:=fList.GetActive;
 If fShuffle
 Then Begin
  If fList.Songs[i].ShufflePos=fList.Count-1 Then Result:=True;
 End
 Else Begin
  If i=fList.Count-1 Then Result:=True;
 End;
end;
{==============================================================================}
{                                    Sort                                      }
{==============================================================================}
procedure TxPlaylist.Sort(SortType:TxSortType;Reverse:boolean);
{------------------------------------------------------------------------------}
Function FormatSong(A:TSong):string;
var i:integer;
    F:string;
begin
 F:=fFormat+' ';
 i:=1; Result:='';
 While i<Length(F) Do
 Begin
  If (F[i]='%') and (Ord(F[i+1])>48) and (Ord(F[i+1])<54)
  Then Begin
        Case Ord(F[i+1]) of
         49: Result:=Result+Trim(A.Artist);
         50: Result:=Result+Trim(A.Title);
         51: Result:=Result+Trim(A.Album);
         52: Result:=Result+Trim(A.Filename);
         53: Result:=Result+Trim(A.Year);
        End;
       Inc(i);
       End
  Else Begin
        Result:=Result+F[i];
       End;
  Inc(i);
 End;
 Result:=Trim(Result);
end;
{------------------------------------------------------------------------------}
Function xMin(A,B:TSong;SortType:TxSortType; Reverse:boolean):boolean;
begin
 Result:=False;
 Case SortType of
  byFilename: If Lowercase(Trim(A.Filename))<Lowercase(Trim(B.Filename)) Then Result:=True Else Result:=False;
  byTitle: If Lowercase(FormatSong(A))<Lowercase(FormatSong(B)) Then Result:=True Else Result:=False;
  byTime: If A.Duration < B.Duration Then Result:=True Else Result:=False;
 End;
 If Reverse Then Result:=not Result;
end;
{------------------------------------------------------------------------------}
var i,j:integer;
    aList:TSongList;
    Flag:boolean;
begin
 aList:=TSongList.Create;
 For i:=0 To fList.Count-1 Do
 Begin
  j:=0; Flag:=False;
  While j<aList.Count
  Do If xMin(fList.Songs[i]^,aList.Songs[j]^,SortType,Reverse)
  Then Begin aList.Insert(j,fList.Songs[i]^); Flag:=True; Break; End
  Else Begin inc(j) End;
  If not Flag Then aList.Add(fList.Songs[i]^);
 End;
 fList.Clear; fPos:=0;
 For i:=0 To aList.Count-1 Do fList.Add(aList.Songs[i]^);
 aList.Free; fList.UnselectAll;
 Paint;
end;
{==============================================================================}
procedure TxPlaylist.RandomizeList;
var i,n1,n2:integer;
begin
 Randomize;
 For i:=0 To (fList.Count-1)*2 Do
 Begin
  n1:=Random(fList.Count+1)-1; n2:=Random(fList.Count+1)-1;
  If n1<0 Then n1:=0; If n2<0 Then n2:=0;
  fList.Exchange(n1,n2);
 End;
 Paint;
end;
{==============================================================================}
{                                    Misc.                                     }
{==============================================================================}
procedure TxPlaylist.SetShuffle(b:boolean);
var i,j:integer;
    P:TSmallIntArray;
begin
 fShuffle:=b;
 If (not fShuffle) or (fList.Count<2) Then exit;
 Randomize; P:=TSmallIntArray.Create(0,2);
 For i:=0 To fList.Count-1 Do P.Add(i);

 For i:=0 To fList.Count-1 Do
 Begin
  j:=Random(P.Count);// writeln(i,'-',PSong(P.Items[i]).ShufflePos);
  fList.Songs[i].ShufflePos:=P.Items[j];  P.Delete(j);
 End;

 P.Free;
end;
{==============================================================================}
procedure TxPlaylist.SetTitleFormat(s:string);
begin
 fFormat:=s;
 Paint;
end;
{==============================================================================}
procedure TxPlaylist.SetFont(F:TFont);
begin
 fFont.Assign(F);
 Canvas.Font.Size:=fFont.Size;
 Canvas.Font.Charset:=fFont.Charset;
 Canvas.Font.Name:=fFont.Name;
 Canvas.Font.Style:=fFont.Style;
 Paint;
end;
{==============================================================================}
procedure TxPlaylist.SetNColor(A:TColor);
begin
 fNColor:=A; Paint;
end;
{==============================================================================}
procedure TxPlaylist.SetAColor(A:TColor);
begin
 fAColor:=A; Paint;
end;
{==============================================================================}
procedure TxPlaylist.SetEColor(A:TColor);
begin
 fEColor:=A; Paint;
end;
{==============================================================================}
procedure TxPlaylist.SetTopPos(n:integer);
begin
 If n<0 Then exit; If n>(fList.Count-1) Then exit;
 fPos:=n; Paint;
 //If Assigned(fOnChangePos) Then fOnChangePos(Self);
end;
{==============================================================================}
function TxPlaylist.GetFormatedDuration(ItemIndex:cardinal):string;
var s:string;
   Sec:LongInt;
begin
 Sec:=fList.Songs[ItemIndex].Duration;
 s:=IntToStr(Sec div 60); Result:=s+':';
 s:=IntToStr(Sec-(Sec div 60)*60); If Length(s)<2 Then s:='0'+s; Result:=Result+s;
end;
{==============================================================================}
function TxPlaylist.GetFormatedString(ItemIndex:cardinal):string;
var S,F:string;
    i:integer;
begin
  //      fFormat
  // %1 - Artist  %2 - Title  %3 - Album  %4 - Filename  %5 - Year

  If (fList.Songs[ItemIndex].Artist='#untagged#') or
     (fList.Songs[ItemIndex].Artist='#untagged_mpeg#')
  Then Begin
   Result:=' '+IntToStr(ItemIndex+1)+'. '+Trim(fList.Songs[ItemIndex].Title); exit;
  End;

  F:=fFormat+' ';
  If fShowSongNumbers
  Then S:=' '+IntToStr(ItemIndex+1)+'. '
  Else S:=' ';
  i:=1;
  While i<Length(F)
  Do Begin
     If (F[i]='%') and (Ord(F[i+1])>48) and (Ord(F[i+1])<54)
     Then Begin
          Case Ord(F[i+1]) of
           49: S:=S+Trim(fList.Songs[ItemIndex].Artist);
           50: S:=S+Trim(fList.Songs[ItemIndex].Title);
           51: S:=S+Trim(fList.Songs[ItemIndex].Album);
           52: S:=S+Trim(fList.Songs[ItemIndex].Filename);
           53: S:=S+Trim(fList.Songs[ItemIndex].Year);
          End;
         Inc(i);
     End
     Else Begin
        S:=S+F[i];
     End;
  Inc(i);
  End;
  Result:=S;
end;
{==============================================================================}
procedure TxPlaylist.GetPlaylistDuration(var TotalDuration, SelDuration:string);
{------------------------------------------------------------------------------}
function FormatTime(Sec:LongInt):string;
var s,m,h:longint;
begin
  h:=Sec div 3600; m:=(Sec div 60)-h*60; s:=Sec-h*3600-m*60;
 Result:=IntToStr(h)+':';
 If Length(IntToStr(m))<2 Then Result:=Result+'0'; Result:=Result+IntToStr(m)+':';
 If Length(IntToStr(s))<2 Then Result:=Result+'0'; Result:=Result+IntToStr(s);
end;
{------------------------------------------------------------------------------}
var i,td,sd:LongInt;
begin
 td:=0; sd:=0;
 For i:=0 To fList.Count-1 Do
 Begin
  td:=td+fList.Songs[i].Duration;
  If fList.Songs[i].Selected Then sd:=sd+fList.Songs[i].Duration;
 End;
 TotalDuration:=FormatTime(td); SelDuration:=FormatTime(sd);
end;
{==============================================================================}
procedure TxPlaylist.RefreshDuration;
var td,sd:string;
begin
 GetPlaylistDuration(td,sd); If Assigned(fOnChangeDuration) Then fOnChangeDuration(td,sd);
end;
{==============================================================================}
procedure TxPlaylist.SetShowSongNumbers(b:boolean);
begin
 fShowSongNumbers:=b; Paint;
end;
{==============================================================================}
procedure Register;
begin
 RegisterComponents('XRat', [TxPlaylist]);
end;
{==============================================================================}


end.
